// ====================================================================
// Copyright (c) 2004 Paul Forgey
// original file Copyright (c) 2002 - 2004 Meedio, LLC.  All rights reserved.
//
// File          : KrellDevice.cpp
// Origin Date   : 11/06/2004
// derived from DeviceTemplate.cpp 9/8/2002
// Purpose       : Main DLL entry point.  Contains the plugin exports
//                   that are called by the HouseBot server.
// ====================================================================

#include "StdAfx.h"
#include "../syCommon/DeviceAPI.h"
#include "KrellModule.h"

#include "resource.h"

static class DeviceModule *module;
static HINSTANCE dllInstance;

static INT_PTR CALLBACK AboutProc (HWND, UINT, WPARAM, LPARAM);

// ====================================================================
// Method Name : DllMain
// Description : Standard Windows DLL entry point.
// ====================================================================
BOOL APIENTRY DllMain( HINSTANCE instance, DWORD  ul_reason_for_call, LPVOID lpReserved )
{
   switch (ul_reason_for_call)
   {
      case DLL_PROCESS_ATTACH:
        dllInstance = instance;
        break;
        
      case DLL_THREAD_ATTACH:
      case DLL_THREAD_DETACH:
         break;

      case DLL_PROCESS_DETACH:
        delete module;
         break;
   }
   return (TRUE);
}

// ====================================================================
// Method Name : HBDeviceModuleInit
// Description : This function is called by the server only once on
//                initial load of the plugin.  It requests information
//                about the plugin and provides server callback
//                information.
// Parameters  :  DeviceCallBackInfo* - A pointer to a DeviceCallBackInfo
//                   structure.  This structure includes the server version
//                   number, the module handle, full path and filename where
//                   the plugin was loaded, and pointers to all server
//                   callback functions.  This structure should be stored and
//                   used for all instances of the Device.
//                   !!== Do not store the pointer to the structure.  Make a
//                   copy of the data.==!!
//                ModuleInfo** - A pointer to a pointer to allow the plugin to
//                   return to the server a ModuleInfo structure that gives
//                   general information about the plugin.  The m_nModuleInfoSize
//                   member must be initialized to the sizeof() the structure.
//                   This allows for backward compatibility if the structure is
//                   amended in the future.
// Returns     : Returns TRUE if successful.  Returns FALSE if unsuccessful.
// ====================================================================
HOUSEBOT_DEVICE_API BOOL HBDeviceModuleInit( const DeviceCallBackInfo* pCallBackInfo, const ModuleInfo** ppModuleInfo )
{
    module = new KrellModule (dllInstance, pCallBackInfo, 
            "1.00", "Krell Showcase/HTS 7.1 Module");
    *ppModuleInfo = module->moduleInfo ();
    if (!module->initializeModule ())
    {
        delete module;
        module = NULL;
        return FALSE;
    }
    
    return TRUE;
}

// ====================================================================
// Method Name : HBDeviceRegisterProperties
// Description : The server calls this function to allow the plugin to
//                register properties for a particular instance.  All
//                properties should be registered when this function is
//                called.  Do not register properties before this
//                function is called.  This function is also called on
//                the plugin for each Device reported by
//                HBGetDeviceInfo() regardless of whether an instance
//                is created or not.
// Parameters  :  int nDeviceNumber - Device Number (ordinal) to register
//                   properties for.  This Device Number must match the
//                   Device number definition given in HBGetDeviceInfo().
// Returns     :  Returns TRUE if the instance has successfully registered
//                   its properties.  Returns FALSE if there was an error.
// ====================================================================
HOUSEBOT_DEVICE_API BOOL HBDeviceRegisterProperties( int nDeviceNumber )
{
    if (module->registerProperties (nDeviceNumber))
        return TRUE;
    return FALSE;
}

// ====================================================================
// Method Name : HBGetDeviceInfo
// Description : The server calls this function to query the plugin for
//                information on each of its supported Devices.  This
//                function is called multiple times to individually
//                query each Device definition.
// Parameters  : int nDeviceNumber - Specifies the Device number (ordinal)
//                that the server is requesting information about.  This
//                always starts with 0 and continues to be called with
//                incrementing nDeviceNumbers until FALSE is returned due
//                to no more Devices to report.
//               DeviceInfo** - A pointer to a pointer to allow the plugin
//                to return to the server a DeviceInfo structure that gives
//                specific information about the Device referenced by
//                nDeviceNumber.  The m_nDeviceInfoSize member must be
//                initialized to the sizeof() the structure.  This allows
//                for backward compatibility if the structure is amended
//                in the future.
// Returns     : Returns TRUE if the device number exists.
//                Returns FALSE if device number does not exist.
// ====================================================================
HOUSEBOT_DEVICE_API BOOL HBGetDeviceInfo( int nDeviceNumber, const DeviceInfo** ppDeviceInfo )
{
    const DeviceInfo *info = module->deviceInfo (nDeviceNumber);
    if (!info)
        return FALSE;
        
    *ppDeviceInfo = info;
    return TRUE;
}


// ====================================================================
// Method Name : HBDeviceInit
// Description : The server calls this function to allow the Device
//                instance to initialize.
// Parameters  : hDevice - Handle to the Device to initialize.
//               bEnabled - Boolean value indicating the startup enabled
//                state of the Device.
//               szDeviceName - Character string indicating the device
//                name that is being initialized.  This is the same name
//                passed in the ModuleInfo structure.  This value allows
//                the plugin to create unique Device objects based on the
//                type/name of the Device.
//               szDeviceDescription - Character string that the user has
//                configured for the Device.  Mainly used to help identify
//                the device when tracing.
// Notes       : This is the first function called by the server that includes
//                the Device Handle (DEVICE_HANDLE).  This is the time to
//                create Device instance objects to maintain
//                per-instance state.
// Returns     : Returns TRUE if successful.  Returns FALSE if unsuccessful.
// ====================================================================
HOUSEBOT_DEVICE_API BOOL HBDeviceInit( DEVICE_HANDLE hDevice, BOOL bEnabled, const char* szDeviceName, const char* szDeviceDescription )
{
    if (module->initialize (hDevice, bEnabled ? true : false, szDeviceName,
            szDeviceDescription))
        return TRUE;
        
    return FALSE;
}

// ====================================================================
// Method Name : HBDeviceSetEnabled
// Description : The server calls this function to indicate to the Device
//                that the user has changed the enabled state of the Device.
// Parameters  : hDevice - Device Handle
//               bEnabled - Boolean value indicating the new Enabled state.
// Notes       : The Device should honor the enabled state.  Typically this
//                means to ignore changes from Property Values when in a
//                disabled state.  It is best to process property value
//                updates, but NOT mark them as complete.  It is possible,
//                however, to process some property values and not others
//                in this state.  For example, the Device may wish to handle
//                non-critical settings in a disabled state, but ignore
//                more 'active' property value changes.
// Returns     :  Returns TRUE if successful.  Returns FALSE if unsuccessful.
// ====================================================================
HOUSEBOT_DEVICE_API BOOL HBDeviceSetEnabled( DEVICE_HANDLE hDevice, BOOL bEnabled )
{
    if (module->setEnabled (hDevice, bEnabled ? true : false))
        return TRUE;
    return FALSE;
}

// ====================================================================
// Method Name : HBDeviceRun
// Description : The server calls this function to allow the Device to run.
// Parameters  : hDevice - Device Handle
//               hTerminate - Windows Event handle that will be set when the
//                server requires the Device to terminate.
//               hPropertyChangeRequest - Windows Event handle that will be
//                set when a property value that is managed by the Device
//                has changed.
// Notes       : The HBDeviceRun() function should not return until the
//                hTerminate event has been set by the server.  The server
//                will create a thread for each Device instance and call
//                HBDeviceRun() expecting the Device to block (using
//                WaitForMultipleObjects()).  If hTermiate is set, the Device
//                should cleanup and return.  If hPropertyChangeRequest is set,
//                the Device should retrieve and process the next property change
//                request in the Device queue.  The Device can also wait on it's
//                own Events and do it's own internal work.
// Returns     : Returns TRUE if successful.  Returns FALSE if unsuccessful.
// ====================================================================
HOUSEBOT_DEVICE_API BOOL HBDeviceRun( DEVICE_HANDLE hDevice, HANDLE hTerminate, HANDLE hPropertyChangeRequest )
{
    if (module->run (hDevice, hTerminate, hPropertyChangeRequest))
        return TRUE;
    return FALSE;
}

// ====================================================================
// Method Name : HBDisplayHelp
// Description : The server calls this function when the user requests help.
// Parameters  : szDeviceName - Name of Device.  If the Plugin supports multiple
//                Devices, this name is used to determine which Device
//                help should be displayed.
// Notes       : The HBDisplayHelp() function should display help for
//                the plugin.  The help can be in any form, but it is
//                suggested, for consistency, that the help be in
//                HTMLHelp format.
// ====================================================================
#if 0
HOUSEBOT_DEVICE_API void HBDisplayHelp( const char* szDeviceName )
{
}
#endif

// ====================================================================
// Method Name : HBAboutBox
// Description : The server calls this function when the user presses the about box button.
// Parameters  : szDeviceName - Name of Device.  If the Plugin supports multiple
//                Devices, this name is used to determine which Device
//                about box should be displayed.
// ====================================================================
HOUSEBOT_DEVICE_API void HBAboutBox( const char* /*szDeviceName*/ )
{
    DialogBox (dllInstance, MAKEINTRESOURCE (IDD_ABOUT), NULL, AboutProc);
}

// ====================================================================
// Method Name : HBDeviceButtonPressNotification
// Description : The server calls this function when the user presses a
//                Property Button.
// Parameters  : hDevice - Device Handle
//               nButtonID - Unique button ID that was passed in pHB_RegisterDeviceButton()
// ====================================================================
HOUSEBOT_DEVICE_API void HBDeviceButtonPressNotification( DEVICE_HANDLE hDevice, int nButtonID )
{
    module->buttonPress (hDevice, nButtonID);
}

// ====================================================================
// Method Name : HBDeviceSubscriptionNotification
// Description : This method is called when any Hardware Interface sends
//                a subscription notification that matches the list name
//                and filter previously registered by this device.
// Parameters  : szListName - Name of the list.  This will match exactly the name of the list
//                as it was registered by the Device.  If subscribing to multiple lists, this
//                name can be used to differentiate between the different notifications.
//               szFilter - Filter that was used by the device to register the list.
//               pDataPack - Pointer to a CDataPack object that was sent from the Hardware Interface.
//                The data that is in the list is specific to the list itself.
// ====================================================================
HOUSEBOT_DEVICE_API BOOL HBDeviceSubscriptionNotification( DEVICE_HANDLE hDevice, LPCTSTR szListName, LPCTSTR szFilter, const CDataPack* pDataPack )
{
    if (module->subscriptionNotification (hDevice, szListName, 
            szFilter ? std::string (szFilter) : std::string (),
            pDataPack))
        return TRUE;
    return FALSE;
}

INT_PTR CALLBACK AboutProc (HWND wnd, UINT msg, WPARAM wparam, LPARAM lparam)
{
    switch (msg)
    {
        case WM_COMMAND:
            switch (LOWORD (wparam))
            {
                case IDOK:
                    EndDialog (wnd, 0);
                    return TRUE;
            }
    }
    return FALSE;
}
