// (c)2004 Paul Forgey
#include "stdafx.h"
#include "KrellHardwareDevice.h"
#include "KrellModule.h"
#include "../../syCommon/DataPack.h"

#include <cstdlib>
#include <cstdio>

CKrellHardwareDevice::CKrellHardwareDevice (DEVICE_HANDLE device, 
        const DeviceModule *module, bool enabled, 
        const std::string &deviceName, const std::string &deviceDescription,
        int model) :
    CDevice (device, module, enabled, deviceName, deviceDescription),
    m_model (model)
{
    std::memset (&m_status, 0, sizeof (m_status));
    std::memset (&m_priorStatus, 0, sizeof (m_priorStatus));
}

CKrellHardwareDevice::~CKrellHardwareDevice ()
{
}

bool CKrellHardwareDevice::DeviceInit (void)
{
    struct PropertyNameAndHandler {
        const char *name;
        HandlerProc handler;
    };
    
    static const struct PropertyNameAndHandler properties[] = {
        { kKrellProperty_DeviceSelect, 
            &CKrellHardwareDevice::changeDeviceSelect },
        { kKrellProperty_VolumeLevel,  
            &CKrellHardwareDevice::changeVolumeLevel },
        { kKrellProperty_CenterTrim,
            &CKrellHardwareDevice::changeTrim },
        { kKrellProperty_SurroundTrim,
            &CKrellHardwareDevice::changeTrim },
        { kKrellProperty_BackTrim,
            &CKrellHardwareDevice::changeTrim },
        { kKrellProperty_SubwooferTrim,
            &CKrellHardwareDevice::changeTrim },
        { kKrellProperty_Balance,
            &CKrellHardwareDevice::changeBalance },
        { kKrellProperty_Compression,
            &CKrellHardwareDevice::changeCompression },
        { kKrellProperty_DirectAudio,
            &CKrellHardwareDevice::changeDirectAudio },
        { kKrellProperty_DirectVideo,
            &CKrellHardwareDevice::changeDirectVideo },
        { kKrellProperty_Power,
            &CKrellHardwareDevice::changePower },
        { kKrellProperty_RoomEQ,
            &CKrellHardwareDevice::changeRoomEQ },
        { kKrellProperty_AudioDecode,
            &CKrellHardwareDevice::changeAudioDecode },
        { kKrellProperty_Trigger1,
            &CKrellHardwareDevice::changeTrigger },
        { kKrellProperty_Trigger2,
            &CKrellHardwareDevice::changeTrigger },
        { kKrellProperty_Trigger3,
            &CKrellHardwareDevice::changeTrigger },
        { kKrellProperty_Trigger4,
            &CKrellHardwareDevice::changeTrigger },
        { kKrellProperty_Muting,
            &CKrellHardwareDevice::changeMuting },
        { kKrellProperty_Button,
            &CKrellHardwareDevice::changeButton },
        { kKrellProperty_CurrentAudioDecode, NULL },
        { kKrellProperty_InputTrigger1, NULL },
        { kKrellProperty_Mode1, NULL },
        { NULL, NULL }
    };

    static const struct PropertyNameAndHandler HTS71properties [] = {
        { kKrellProperty_MainDeviceSelect,
            &CKrellHardwareDevice::changeDeviceSelect },
        { kKrellProperty_Zone2DeviceSelect,
            &CKrellHardwareDevice::changeDeviceSelect },
        { kKrellProperty_Zone2VolumeLevel,
            &CKrellHardwareDevice::changeVolumeLevel },
        { kKrellProperty_MainDirectAudio,
            &CKrellHardwareDevice::changeDirectAudio },
        { kKrellProperty_Zone2DirectAudio,
            &CKrellHardwareDevice::changeDirectAudio },
        { kKrellProperty_Zone2Power,
            &CKrellHardwareDevice::changePower },
        { kKrellProperty_Zone2Muting,
            &CKrellHardwareDevice::changeMuting },
        { kKrellProperty_CurrentZone, NULL },
        { NULL, NULL }
    };

    const struct PropertyNameAndHandler *i;
    std::string propertyName;
    CProperty *property;
    
    for (i = &properties[0]; i->name != NULL; ++i)
    {
        propertyName = i->name;
        property = CreatePropertyByName (propertyName);
        m_properties[propertyName] = property;
        m_handlers[property] = i->handler;
    }
    
    if (m_model == KrellModule::kDevice_HTS71)
    for (i = &HTS71properties[0]; i->name != NULL; ++i)
    {
        propertyName = i->name;
        property = CreatePropertyByName (propertyName);
        m_properties[propertyName] = property;
        m_handlers[property] = i->handler;
    }
    
    return true;
}

void CKrellHardwareDevice::Command (const char *string)
{
    InterfaceArgumentPack pack (kKrellSignature_Command, 1);
    pack[0].stringValue (string);

    SendInterfacePackToInterface (&pack);
}

bool CKrellHardwareDevice::changePropertyValue (CProperty *property, 
        const std::string &newValue, bool allowSameValue)
{
    if (!property)
        return false;

    if (!allowSameValue && property->GetPropertyValueAsString () == newValue)
        return true;

    CPropertyValue *value = property->CreatePropertyValue (newValue);
    if (value)
    {
        value->ChangeRequestCompleted ();
        delete value;
        return true;
    }
    return false;
}

void CKrellHardwareDevice::audioInputChanged (CProperty *prop, BYTE input)
{
    if (input)
        changePropertyValue (prop, GetDeviceModule ()->directAudio (input-1,
            m_model), false);
}

void CKrellHardwareDevice::trimChanged (CProperty *prop, BYTE value)
{
    char buf[33];
    std::sprintf (buf, "%d", value-10);
    changePropertyValue (prop, buf, FALSE);
}

void CKrellHardwareDevice::balanceChanged (BYTE value)
{
    int balance = (value & kKrellBalance_Mask);
    int compression = ((value & ~kKrellBalance_Mask) >> 5) -1;
    char buf[33];
    
    std::sprintf (buf, "%d", balance-13);

    changePropertyValue (m_properties[kKrellProperty_Balance], buf, FALSE);
        
    const char *compstr = GetDeviceModule ()->compression (compression);
    changePropertyValue (m_properties[kKrellProperty_Compression], compstr, 
            FALSE);
}

void CKrellHardwareDevice::modeLEDChanged (BYTE led)
{
    changePropertyValue (m_properties[kKrellProperty_Mode1], 
            (led & kKrellLED_Mode1) ? "On" : "Off", FALSE);
}

void CKrellHardwareDevice::decodeChanged (BYTE decode)
{
    changePropertyValue (m_properties[kKrellProperty_CurrentAudioDecode], 
            GetDeviceModule ()->currentAudioDecode (decode), FALSE);
}

void CKrellHardwareDevice::inputChanged (CProperty *prop, BYTE input)
{
    changePropertyValue (prop, GetDeviceModule ()->device (input, m_model),
            false);
}

void CKrellHardwareDevice::eqChanged (BYTE byte)
{
    int zone2Balance = (byte & kKrellBalance_Mask);
    int eq = (int) (byte & ~kKrellBalance_Mask) >> 5;

    char buf[33];
    std::sprintf (buf, "%d", eq);
    changePropertyValue (m_properties[kKrellProperty_RoomEQ], buf, FALSE);
        
    std::sprintf (buf, "%d", zone2Balance-13);
    changePropertyValue (m_properties[kKrellProperty_Zone2Balance], buf, 
            FALSE);
}

void CKrellHardwareDevice::statusChanged (BYTE s1, BYTE s2)
{
    CProperty *power = m_properties[kKrellProperty_Power];
    
    if (m_model == KrellModule::kDevice_Showcase)
        changePropertyValue (power, (s1 & kKrellStatus1_Power) ? 
                "On" : "Off", false);
    else {
        if ((m_status.status1 & 0x3) == kKrellStatus1_Power) // exclude zone2 only from main power
            changePropertyValue (power, "On", false);
        else
            changePropertyValue (power, "Off", false);
    }

    if (m_model == KrellModule::kDevice_HTS71)
        changePropertyValue (m_properties[kKrellProperty_Zone2Power], 
                (s1 & (kKrellStatus1_Zone2Power | kKrellStatus1_Zone2Only)) ? 
                "On" : "Off", false);
    
    changePropertyValue (m_properties[kKrellProperty_Muting], 
            (s1 & kKrellStatus1_MainMute) ? "On" : "Off", false);
    
    if (m_model == KrellModule::kDevice_HTS71)
    {
        changePropertyValue (m_properties[kKrellProperty_Zone2Muting], 
                (s1 & kKrellStatus1_Zone2Mute) ? "On" : "Off", false);

        changePropertyValue (m_properties[kKrellProperty_CurrentZone], 
                (s1 & kKrellStatus1_Zone2Active) ? "Zone2" : "Main", false);
    }

    changePropertyValue (m_properties[kKrellProperty_InputTrigger1], 
        (s1 & kKrellStatus1_InTrigger1) ? "On" : "Off", FALSE);

    changePropertyValue (m_properties[kKrellProperty_Trigger1], 
        (s2 & kKrellStatus2_Trigger1) ? "On" : "Off", FALSE);

    changePropertyValue (m_properties[kKrellProperty_Trigger2], 
        (s2 & kKrellStatus2_Trigger2) ? "On" : "Off", FALSE);

    changePropertyValue (m_properties[kKrellProperty_Trigger3], 
        (s2 & kKrellStatus2_Trigger3) ? "On" : "Off", FALSE);

    changePropertyValue (m_properties[kKrellProperty_Trigger4], 
        (s2 & kKrellStatus2_Trigger4) ? "On" : "Off", FALSE);
}


void CKrellHardwareDevice::streamChanged (BYTE)
{
    // oh who cares
}

void CKrellHardwareDevice::videoInputChanged (BYTE input)
{
    if (input != 0)
        changePropertyValue (m_properties[kKrellProperty_DirectVideo], 
                GetDeviceModule ()->directVideo (input-1), FALSE);
}

void CKrellHardwareDevice::volumeChanged (CProperty *prop, BYTE value)
{
    char buf[33];
    std::sprintf (buf, "%d", (int) value);
    changePropertyValue (prop, buf, false);
}

void CKrellHardwareDevice::changeAudioDecode (CPropertyValue *change)
{
    const char *code = GetDeviceModule ()->audioDecodeCode (
            change->GetPropertyValue ().c_str ());
    if (code != NULL)
    {
        Command (code);
        change->ChangeRequestCompleted ();
    }
    else
    {
        change->ChangeRequestFailed ();
    }
    
    delete change;
}

void CKrellHardwareDevice::changeBalance (CPropertyValue *change)
{
    int ival;
    char buf[32];
    const std::string &propertyName (change->GetOwningProperty ()->
            GetPropertyName ());
    if (propertyName == kKrellProperty_Balance)
    {
        ival = strtol (change->GetPropertyValue ().c_str (), NULL, 10);
        ival += 13;
        if (ival < 0)
            ival = 0;
        else
        if (ival > 26)
            ival = 26;
            
        std::sprintf (buf, "%02BALZ"); 
        Command (buf);
        change->ChangeRequestCompleted ();
    }
    else
    if (propertyName == kKrellProperty_Zone2Balance)
    {
        // XXX not documented
        change->ChangeRequestFailed ();
    }
    else
        change->ChangeRequestFailed ();
        
    delete change;
}

void CKrellHardwareDevice::changeButton (CPropertyValue *change)
{
    const char *code = GetDeviceModule ()->buttonCode (
            change->GetPropertyValue ().c_str ());
    if (code)
    {
        const CProperty *owningProperty = change->GetOwningProperty ();
    
        Command (code);
        change->ChangeRequestCompleted ();
        delete change;
        
        change = owningProperty->CreatePropertyValue ("None");
        change->ChangeRequestCompleted ();
    }
    else
        change->ChangeRequestFailed ();

    delete change;
}

void CKrellHardwareDevice::changeCompression (CPropertyValue *change)
{
    const char *code = GetDeviceModule ()->compressionCode (change->
            GetPropertyValue ().c_str ());
    if (code)
    {
        Command (code);
        change->ChangeRequestCompleted ();
    }
    else
        change->ChangeRequestFailed ();
        
    delete change;
}

void CKrellHardwareDevice::changeDeviceSelect (CPropertyValue *change)
{
    const std::string &propertyName = change->GetOwningProperty ()->
            GetPropertyName ();
    const char *code;
    if (propertyName == kKrellProperty_DeviceSelect)
        code = GetDeviceModule ()->deviceCode (
                change->GetPropertyValue ().c_str (),
                KrellModule::kZone_Current);
    else
    if (propertyName == kKrellProperty_MainDeviceSelect)
        code = GetDeviceModule ()->deviceCode (
                change->GetPropertyValue ().c_str (),
                KrellModule::kZone_Main);
    else        
    if (propertyName == kKrellProperty_Zone2DeviceSelect)
        code = GetDeviceModule ()->deviceCode (
                change->GetPropertyValue ().c_str (),
                KrellModule::kZone_Secondary);
    else
        code = NULL;
        
    if (code)
    {
        Command (code);
        change->ChangeRequestCompleted ();
    }
    else
        change->ChangeRequestFailed ();
        
    delete change;
}

void CKrellHardwareDevice::changeDirectAudio (CPropertyValue *change)
{
    const char *code;
    const std::string &propertyName = change->GetOwningProperty ()->
            GetPropertyName ();
    if (propertyName == kKrellProperty_DirectAudio)
        code = GetDeviceModule ()->directAudioCode (
                change->GetPropertyValue ().c_str (), 
                KrellModule::kZone_Current);
    else
    if (propertyName == kKrellProperty_MainDirectAudio)
        code = GetDeviceModule ()->directAudioCode (
                change->GetPropertyValue ().c_str (),
                KrellModule::kZone_Main);
    else
    if (propertyName == kKrellProperty_Zone2DirectAudio)
        code = GetDeviceModule ()->directAudioCode (
                change->GetPropertyValue ().c_str (),
                KrellModule::kZone_Secondary);
    else
        code = NULL;
        
    if (code)
    {
        Command (code);
        change->ChangeRequestCompleted ();
    }
    else
        change->ChangeRequestAbandoned ();
        
    delete change;
}

void CKrellHardwareDevice::changeDirectVideo (CPropertyValue *change)
{
    const char *code = GetDeviceModule ()->directVideoCode (
                change->GetPropertyValue ().c_str ());
    if (code)
    {
        Command (code);
        change->ChangeRequestCompleted ();
    }
    else
        change->ChangeRequestFailed ();
        
    delete change;
}

void CKrellHardwareDevice::changeMuting (CPropertyValue *change)
{
    const std::string &propertyName = change->GetOwningProperty ()->
            GetPropertyName ();
    const char *code;
    bool mute = false;
    if (propertyName == kKrellProperty_Muting)
    {
        if (change->GetPropertyValue () == "On")
            mute = true;
            
        if ((m_status.status1 & kKrellStatus1_MainMute) != (mute ? kKrellStatus1_MainMute : 0))
            code = m_model == KrellModule::kDevice_Showcase ? "MUTZ" : "MUTMZ";
        else
        {
            // blow it off successfully
            change->ChangeRequestCompleted ();
            delete change;
            return;
        }
    }
    else
    if (propertyName == kKrellProperty_Zone2Muting)
    {
        if (change->GetPropertyValue () == "On")
            mute = true;
            
        if ((m_status.status1 & kKrellStatus1_Zone2Mute) != (mute ? kKrellStatus1_Zone2Mute : 0))
            code = "MUTRZ";
        else
        {
            change->ChangeRequestCompleted ();
            delete change;
            return;
        }
    }
    else
        code = NULL;
        
    if (code)
    {
        Command (code);
        change->ChangeRequestCompleted ();
    }
    else
        change->ChangeRequestFailed ();
        
    delete change;
}

void CKrellHardwareDevice::changePower (CPropertyValue *change)
{
    const char *code;
    const std::string &propertyName = change->GetOwningProperty ()->
            GetPropertyName ();
    if (propertyName == kKrellProperty_Power)
    {
        if (change->GetPropertyValue () == "On")
            code = "1PWRZ";
        else
            code = "0PWRZ";
    }
    else
    if (propertyName == kKrellProperty_Zone2Power)
    {
        if (change->GetPropertyValue () == "Off")
        {
            if (m_status.status1 & kKrellStatus1_Zone2Active)
            {
                change->ChangeRequestCompleted ();
                delete change;
                return;
            }
            
            code = "REMZ";
        }
        else
        {
            if (!(m_status.status1 & (kKrellStatus1_Zone2Power | kKrellStatus1_Zone2Only)))
            {
                change->ChangeRequestCompleted ();
                delete change;
                return;
            }
            
            code = "REMZ";
        }            
    }
    else
        code = NULL;
        
    if (code)
    {
        Command (code);
        change->ChangeRequestCompleted ();
    }
    else
        change->ChangeRequestFailed ();
        
    delete change;
}

void CKrellHardwareDevice::changeRoomEQ (CPropertyValue *change)
{
    char buf[32];
    int ival = std::strtol (change->GetPropertyValue ().c_str (), NULL, 10);
    if (ival < 0)
        ival = 0;
    else
    if (ival > 4)
        ival = 4;
            
    std::sprintf (buf, "%03dEQMZ");
    Command (buf);
    change->ChangeRequestCompleted ();
    delete change;
}

void CKrellHardwareDevice::changeTrigger (CPropertyValue *change)
{
    const char *code = NULL;
    const std::string &propertyName = change->GetOwningProperty ()->
            GetPropertyName ();
            
    if (propertyName == kKrellProperty_Trigger1)
    { 
        if (change->GetPropertyValue () == "On")
            code = "1TR1Z";
        else
            code = "0TR1Z";
    }
    else
    if (propertyName == kKrellProperty_Trigger2)
    {
        if (change->GetPropertyValue () == "On")
            code = "1TR2Z";
        else
            code = "0TR2Z";
    }
    else
    if (propertyName == kKrellProperty_Trigger3)
    {
        if (change->GetPropertyValue () == "On")
            code = "1TR3Z";
        else
            code = "0TR3Z";
    }
    else
    if (propertyName == kKrellProperty_Trigger4)
    {
        if (change->GetPropertyValue () == "On")
            code = "1TR4Z";
        else
            code = "0TR4Z";
    }
    
    if (code)
    {
        Command (code);
        change->ChangeRequestCompleted ();
    }
    else
        change->ChangeRequestAbandoned ();
        
    delete change;
}

void CKrellHardwareDevice::changeTrim (CPropertyValue *change)
{
    int ival;
    char buf[32];
    const std::string &propertyName = change->GetOwningProperty ()->
            GetPropertyName ();
    if (propertyName == kKrellProperty_CenterTrim)
    {
        ival = std::strtol (change->GetPropertyValue ().c_str (), NULL, 10);
        ival += 10;
        if (ival < 0)
            ival = 0;
        else
        if (ival > 20)
            ival = 20;
        
        std::sprintf (buf, "%03dCVLZ", ival);
    }
    else
    if (propertyName == kKrellProperty_SurroundTrim)
    {
        ival = std::strtol (change->GetPropertyValue ().c_str (), NULL, 10);
        ival += 10;
        if (ival < 0)
            ival = 0;
        else
        if (ival > 20)
            ival = 20;
        
        std::sprintf (buf, "%03dSRLZ", ival);
    }
    else
    if (propertyName == kKrellProperty_BackTrim)
    {
        ival = std::strtol (change->GetPropertyValue ().c_str (), NULL, 10);
        ival += 10;
        if (ival < 0)
            ival = 0;
        else
        if (ival > 20)
            ival = 20;
            
        std::sprintf (buf, "%03dBVLZ");
    }
    else
    if (propertyName == kKrellProperty_SubwooferTrim)
    {
        ival = std::strtol (change->GetPropertyValue ().c_str (), NULL, 10);
        ival += 10;
        if (ival < 0)
            ival = 0;
        else
        if (ival > 20)
            ival = 20;
            
        std::sprintf (buf, "%03dSRLZ");
    }
    else
    {
        change->ChangeRequestFailed ();
        delete change;
        return;
    }
    
    Command (buf);
    change->ChangeRequestCompleted ();
    delete change;
}

void CKrellHardwareDevice::changeVolumeLevel (CPropertyValue *change)
{
    int ival;
    char buf[32];
    const std::string &propertyName = change->GetOwningProperty ()->
            GetPropertyName ();
    if (propertyName == kKrellProperty_VolumeLevel)
    {
        // live status will come around and set within range
        ival = std::strtol (change->GetPropertyValue ().c_str (), NULL, 10);
        if (ival < 0)
            ival = 0;
        else
        if (ival > 152)
            ival = 152;
            
        std::sprintf (buf, "%03dMVLZ", ival);
    }
    else
    if (propertyName == kKrellProperty_Zone2VolumeLevel)
    {
        ival = std::strtol (change->GetPropertyValue ().c_str (), NULL, 10);
        if (ival < 0)
            ival = 0;
        else
        if (ival > 152)
            ival = 152;
            
        std::sprintf (buf, "%03dRVLZ", ival);
    }
    else
    {
        change->ChangeRequestFailed ();
        delete change;
        return;
    }
    
    Command (buf);
    change->ChangeRequestCompleted ();
    delete change;
}

void CKrellHardwareDevice::changeProperty (CPropertyValue *change)
{
    if (!GetEnabled ())
        return;

    const CProperty *owningProperty = change->GetOwningProperty ();

    handlermap_type::iterator i = m_handlers.find (owningProperty);
    if (i != m_handlers.end ())
        (this->*(i->second))(change);
    else
        change->ChangeRequestFailed ();    
}

bool CKrellHardwareDevice::DeviceRun (HANDLE terminate, HANDLE changeRequest)
{
    HANDLE waitvec[3] = { terminate, changeRequest };
    DWORD wait;
    CPropertyValue *change;
    BOOL done = FALSE;

    SubscribeToNotificationList( kKrellNotification_Status, "" );
    
    // get an initial status
    Command ("STAZ");
    
    while (!done)
    {
        wait = WaitForMultipleObjects (2, waitvec, FALSE, INFINITE);
        switch (wait)
        {
        case WAIT_OBJECT_0:
            done = TRUE;
            break;
            
        case WAIT_OBJECT_0+1:
            change = GetNextPropertyValueOnChangeRequestQueue ();
            if (change)
                changeProperty (change);
            break;
                
        case WAIT_FAILED:
            TraceMessage (ttError, "WaitForMultipleObjects() failed: %08x", GetLastError ());
            done = TRUE;
            break;
        
        default:
            TraceMessage (ttError, "unexpected WaitForMultipleObjects() result: %08x", wait);
            done = TRUE;
            break;
        }
    }
    
    UnSubscribeFromNotificationList( kKrellNotification_Status, "" );
    return true;
}

void CKrellHardwareDevice::ButtonPressNotification (int)
{
}

bool CKrellHardwareDevice::SubscriptionNotification (const std::string &listName, 
        const std::string &filter, const CDataPack *pack)
{
    if (listName == kKrellNotification_Status)
    {
        int size;
        const KrellStatus *status = reinterpret_cast< const KrellStatus *> (
                pack->GetBinaryData ("Status", &size));
        if (status && size == sizeof (KrellStatus) && 
                status->start == 0x55 && status->end == 0x55)
        {
            TraceMessage (ttDebug, "Got notification");
            
            std::memcpy (&m_status, status, sizeof (m_status));
        
            // handle all the changes from the device
            if (!m_priorStatus.start || 
                    (m_priorStatus.audioInput != m_status.audioInput))
            {
                audioInputChanged (
                    m_properties[m_model == KrellModule::kDevice_HTS71 ? 
                        kKrellProperty_MainDirectAudio : kKrellProperty_DirectAudio],
                        m_status.audioInput);
                if (m_model == KrellModule::kDevice_HTS71)
                    if (!(m_status.status1 & kKrellStatus1_Zone2Active))
                        audioInputChanged (m_properties[kKrellProperty_DirectAudio],
                            m_status.audioInput); // current zone
            }
            
            if (m_model == KrellModule::kDevice_HTS71)
            {
                if (!m_priorStatus.start || 
                        (m_priorStatus.zone2Input != m_status.zone2Input))
                    audioInputChanged (
                        m_properties[kKrellProperty_Zone2DirectAudio], 
                        m_status.zone2Input);

                if (m_status.status1 & kKrellStatus1_Zone2Active)
                    audioInputChanged (
                        m_properties[kKrellProperty_DirectAudio], 
                        m_status.zone2Input); // current zone
            }
            
            if (!m_priorStatus.start || ((m_priorStatus.backTrim & kKrellTrim_Mask) !=
                                            (m_status.backTrim & kKrellTrim_Mask)))
                trimChanged (m_properties[kKrellProperty_BackTrim],
                        m_status.backTrim & kKrellTrim_Mask);
                
            if (!m_priorStatus.start || (m_priorStatus.balance != m_status.balance))
                balanceChanged (m_status.balance);
                
            if (!m_priorStatus.start || ((m_priorStatus.surroundTrim & kKrellLED_Mode1) !=
                                            (m_status.surroundTrim & kKrellLED_Mode1)))
                modeLEDChanged (m_status.surroundTrim & kKrellLED_Mode1);
                
            if (!m_priorStatus.start || ((m_priorStatus.centerTrim & kKrellTrim_Mask) !=
                                            (m_status.centerTrim & kKrellTrim_Mask)))
                trimChanged (m_properties[kKrellProperty_CenterTrim], 
                        m_status.centerTrim & kKrellTrim_Mask);
                
            if (!m_priorStatus.start || (m_priorStatus.decode != m_status.decode))
                decodeChanged (m_status.decode);
                    
            if (!m_priorStatus.start || ((m_priorStatus.device & 0x0f) != (m_status.device & 0x0f)))
                inputChanged (m_properties[kKrellProperty_DeviceSelect], 
                        m_status.device & 0x0f);

            if (m_model == KrellModule::kDevice_HTS71 && (!m_priorStatus.start || 
                    ((m_priorStatus.device & 0xf0) != (m_status.device & 0xf0))))
                inputChanged (m_properties[kKrellProperty_Zone2DeviceSelect], 
                        (m_status.device & 0xf0) >> 4);
                
            if (!m_priorStatus.start || (m_priorStatus.eq != m_status.eq))
                eqChanged (m_status.eq);
                
            if (!m_priorStatus.start || (m_priorStatus.status1 != m_status.status1) ||
                                        (m_priorStatus.status2 != m_status.status2))
                statusChanged (m_status.status1, m_status.status2);
            
            if (!m_priorStatus.start || (m_priorStatus.stream != m_status.stream))
                streamChanged (m_status.stream);
                
            if (!m_priorStatus.start || ((m_priorStatus.subTrim & kKrellTrim_Mask) !=
                                            (m_status.subTrim & kKrellTrim_Mask)))
                trimChanged (m_properties[kKrellProperty_SubwooferTrim], 
                        m_status.subTrim & kKrellTrim_Mask);
                
            if (!m_priorStatus.start || ((m_priorStatus.surroundTrim & kKrellTrim_Mask) !=
                                            (m_status.surroundTrim & kKrellTrim_Mask)))
                trimChanged (m_properties[kKrellProperty_SurroundTrim], 
                        m_status.surroundTrim & kKrellTrim_Mask);
                
            if (!m_priorStatus.start || (m_priorStatus.videoInput != m_status.videoInput))
                videoInputChanged (m_status.videoInput);
                
            if (!m_priorStatus.start || (m_priorStatus.volume != m_status.volume))
                volumeChanged (m_properties[kKrellProperty_VolumeLevel], 
                        m_status.volume);
                
            if (m_model == KrellModule::kDevice_HTS71 &&
                (!m_priorStatus.start || 
                        (m_priorStatus.zone2Volume != m_status.zone2Volume)))
                    volumeChanged (m_properties[kKrellProperty_Zone2VolumeLevel], 
                            m_status.zone2Volume);
                            
            // please make a note of it
            std::memcpy (&m_priorStatus, &m_status, sizeof (m_priorStatus));
            // reset status
            m_status.start = m_status.end = 0;
        }
    }

    return true;
}

