// Copyright (c)2004 Paul Forgey

#include "StdAfx.h"
#include "KrellModule.h"
#include "KrellHardwareDevice.h"
#include "KrellProperties.h"

#include <cstring>
#include <algorithm>
#include <functional>

struct KrellProperty {
    int                 deviceNumber;   // -1 for all
    const char          *name;
    const char          *initialValue;
    eIOType             ioType;
    ePropertyType       type;
    const char *const   *firstValue;
    const char *const   *lastValue;                
};

static const char *kInvalid = "<invalid>";

static const char *kCompression [] = { 
    "Max\0MXCZ",
    "Normal\0NMCZ", 
    "Night\0NTCZ"
};

static const char *kDevices [] = {
    "DVD\0DVDZ", 
    "LD\0LDZ", 
    "Satellite\0SATZ", 
    "VCR\0VCRZ", 
    "TV\0TVZ", 
    "CD\0CDZ", 
    "Tuner\0TUNZ", 
    "Aux1\0AU1Z", 
    "Game\0AU2Z", 
    "Tape\0TPZ",
    // to allow looking up code for either model
    "Aux2\0AU2Z"
};
static const char *kMainDevices [] = { // HTS71
    "DVD\0DVDMZ", 
    "LD\0LDMZ", 
    "Satellite\0SATMZ", 
    "VCR\0VCRMZ", 
    "TV\0TVMZ", 
    "CD\0CDMZ", 
    "Tuner\0TUNMZ", 
    "Aux1\0AU1MZ", 
    "Aux2\0AU2MZ", 
    "Tape\0TPMZ"
};
static const char *kZone2Devices [] = { // HTS71
    "DVD\0DVDRZ", 
    "LD\0LDRZ", 
    "Satellite\0SATRZ", 
    "VCR\0VCRRZ", 
    "TV\0TVRZ", 
    "CD\0CDRZ", 
    "Tuner\0TUNRZ", 
    "Aux1\0AU1RZ", 
    "Aux2\0AU2RZ", 
    "Tape\0TPRZ"
};

static const char *kDirectShowcaseAudio[] = {  // showcase
    "Coax 1\0CO1Z",
    "Coax 2\0CO2Z",
    "Coax 3\0CO3Z",
    "Coax 4\0CO4Z",
    "Opt 1\0OPT1Z",
    "Opt 2\0OPT2Z",
    "Opt 3\0OPT3Z",
    "Opt 4\0OPT4Z",
    "Balanced 1\0AB1Z",
    "Stereo 1\0AS1Z",
    "Stereo 2\0AS2Z",
    "Stereo 3\0AS3Z",
    "Stereo 4\0AS4Z",
    "Stereo 5\0AS5Z",
    "Stereo 6\0AS6Z",
    "Stereo 7\0AS7Z",
    "Tape\0ATPAZ",
    "VCR\0AVCAZ",
    "7.1\0AMCZ", // undocumented but valid for input status
    // to allow looking up code for either model
    "Coax 5\0CO5Z",
    "Coax 6\0CO6Z"        
};
static const char *kDirectHTS71Audio[] = {  // HTS 71 current zone
    "Coax 1\0CO1Z",
    "Coax 2\0CO2Z",
    "Coax 3\0CO3Z",
    "Coax 4\0CO4Z",
    "Coax 5\0CO5Z",
    "Coax 6\0CO6Z",
    "Opt 1\0OPT1Z",
    "Opt 2\0OPT2Z",
    "Balanced 1\0AB1Z",
    "Stereo 1\0AS1Z",
    "Stereo 2\0AS2Z",
    "Stereo 3\0AS3Z",
    "Stereo 4\0AS4Z",
    "Stereo 5\0AS5Z",
    "Stereo 6\0AS6Z",
    "Stereo 7\0AS7Z",
    "Tape\0ATPAZ",
    "VCR\0AVCAZ",
    "7.1\0AMCZ" // undocumented but valid for input status
};
static const char *kDirectMainAudio [] = { // HTS71
    "Coax 1\0CO1MZ",    // There are no documented direct 
    "Coax 2\0CO2MZ",    // main zone digital audio inputs.
    "Coax 3\0CO3MZ",    // These are a logical guess, because
    "Coax 4\0CO4MZ",    // it would be highly surprising to
    "Coax 5\0CO5MZ",    // not have them. More important these
    "Coax 6\0CO6MZ",    // are also here for status display.
    "Opt 1\0OPT1MZ",
    "Opt 2\0OPT2MZ",
    "Balanced 1\0AB1MZ",
    "Stereo 1\0AS1MZ",
    "Stereo 2\0AS2MZ",
    "Stereo 3\0AS3MZ",
    "Stereo 4\0AS4MZ",
    "Stereo 5\0AS5MZ",
    "Stereo 6\0AS6MZ",
    "Stereo 7\0AS7MZ",
    "Tape\0TPAMZ",
    "VCR\0VCAMZ",
    "7.1\0AMCMZ"       // valid input, but just a guess for output
};
static const char *kDirectZone2Audio [] = { // HTS71
    "Coax 1\0CO1RZ",
    "Coax 2\0CO2RZ",
    "Coax 3\0CO3RZ",
    "Coax 4\0CO4RZ",
    "Coax 5\0CO5RZ",
    "Coax 6\0CO6RZ",
    "Opt 1\0OPT1RZ",
    "Opt 2\0OPT2RZ",
    "Balanced 1\0AB1RZ",
    "Stereo 1\0AS1RZ",
    "Stereo 2\0AS2RZ",
    "Stereo 3\0AS3RZ",
    "Stereo 4\0AS4RZ",
    "Stereo 5\0AS5RZ",
    "Stereo 6\0AS6RZ",
    "Stereo 7\0AS7RZ",
    "Tape\0TPARZ",
    "VCR\0VCARZ",
    "7.1\0AMCRZ"    // well, not really (placeholder)
};

static const char *kDirectVideo [] = {
    "S-Video 1\0SV1Z",
    "S-Video 2\0SV2Z",
    "S-Video 3\0SV3Z",
    "S-Video 4\0SV4Z",
    "Component 1\0CM1Z",
    "Component 2\0CM2Z",
    "Component 3\0CM3Z",    // showcase only
    "Composite 1\0CS1Z",
    "Composite 2\0CS2Z",
    "Composite 3\0CS3Z",
    "Composite 4\0CS4Z"
};

// may I never again have to type in all these audio modes!

static const char *kAudioDecode [] = {
    // map to the remote and front panel buttons.  some of these cycle further modes
    "Mode 1\0MS1Z",
    "Mode 2\0MS2Z",
    "Pro-Logic\0MVZ",
    "Stereo\0STZ",
    "Preamp\0PAZ",
    "THX\0THXZ", // XXX documentation is wrong, and this doesn't work on a Showcase
    // direct discrete selection
    "Dolby Digital 5.1\000000MODZ",
    "Dolby Digital 5.1 + THX\000001MODZ",
    "Dolby Digital 5.1 + THX EX\000002MODZ",
    "AC3 2/0 Stereo (Pass Mode)\000003MODZ",
    "AC3 2/0 Pro-Logic II Movie\000004MODZ",
    "AC3 2/0 Pro-Logic II Music\00005MODZ",
    "AC3 2/0 Pro-Logic II Matrix\00006MODZ",
    "AC3 2/0 Pro-Logic II Emulation\000007MODZ",
    "AC3 2/0 Pro-Logic II Movie+THX\000008MODZ",
    "AC3 2/0 Pro-Logic II Emulation+THX\000009MODZ",
    "DTS 5.1 Music\000010MODZ",
    "DTS 5.1 Movie\000011MODZ",
    "DTS 5.1 Matrix\000012MODZ",
    "DTS 5.1 Movie + THX\000013MODZ",
    "DTS 5.1 Matrix + THX\000014MODZ",
    "DTS 6.1 Discrete\000015MODZ",
    "DTS 6.1 Discrete + THX\000016MODZ",
    "DTS 5.1 Force\000017MODZ",
    "Stereo Mode (Pass Mode)\000018MODZ",
    "Pro-Logic II Movie\000019MODZ",
    "Pro-Logic II Music\000020MODZ",
    "Pro-Logic II Matrix\000021MODZ",
    "Pro-Logic II Emulation\000022MODZ",
    "Pro-Logic II Movie + THX\000023MODZ",
    "Pro-Logic II Emulation + THX\000024MODZ",
    "DTS Neo:6 Cinema 6\000025MODZ",
    "DTS Neo:6 Music\000026MODZ",
    "DTS Neo:6 Cinema 6 + THX\000027MODZ",
    "Party\000028MODZ",
    "General Admission\000029MODZ",
    "Front Row\000030MODZ",
    "On Stage\000031MODZ",
    "Enhanced Stereo\000032MODZ",
    "Orchestra\000033MODZ",
    "Mezzanine\000034MODZ",
    "Full Range + Sub\000035MODZ",
    "Monophonic\000036MODZ",
    "24/96 Decode\000038MODZ"
};

static const char *kButtons [] = {
    "None\0",
    "Up\0UPZ",
    "Down\0DWNZ",
    "Left\0LFTZ",
    "Right\0RGTZ",
    "Menu\0MENZ",
    "Enter\0ENTZ",
    "Previous\0PRVZ",
    "Center Trim\0ICVLZ",
    "Surround Trim\0IRVLZ",
    "Sub Trim\0ISUBZ",
    "Balance\0IBALZ",
    "Mute\0MUTZ",
    // HTS71 below this point
    "Zone2 Balance\0BRLZ",
    "Main Mute\0MUTMZ",
    "Zone2 Mute\0MUTRZ",
    "Main\0MANZ",
    "Zone2\0REMZ"  // zone2 power toggle if main zone is off
};

static const char *kCurrentAudioDecode [] = {
    kInvalid,                       // 0
    // analog and pcm decode modes
    "Stereo",                       // 1
    "Pro-Logic II Movie",           // 2
    "Pro-Logic II Music",           // 3
    "Pro-Logic II Matrix",          // 4
    "Pro-Logic II Emulation",       // 5
    "Pro-Logic II Movie+THX",       // 6
    "7.1",                          // 7
    "Pro-Logic II Emulation+THX",   // 8
    "DTS Neo:6 Cinema 6",           // 9
    "PCM 24/96",                    // 10
    "DTS Neo:6 Music",              // 11
    "DTS Neo:6 Cinema 6+THX",       // 12
    "Pshhhhh",                      // 13
    "Party",                        // 14
    "General Adminission",          // 15
    "Front Row",                    // 16
    "On Stage",                     // 17
    "Enhanced Stereo",              // 18
    "Orchestra",                    // 19
    "Mezzanine",                    // 20
    "Full Range+Sub",               // 21
    "Monophonic",                   // 22
    "Preamp",                       // 23
    // dolby ac-3 2.0 modes
    "AC3 2.0 Stereo",               // 24
    "AC3 2.0 Pro-Logic II Movie",   // 25
    "AC3 2.0 Pro-Logic II Music",   // 26
    "AC3 2.0 Pro-Logic II Matrix",  // 27
    "AC3 2.0 Pro-Logic II Emulation",   // 28
    "AC3 2.0 Pro-Logic II Movie+THX",   // 29
    "AC3 2.0 Pro-Logic II Matrix+THX",  // 30
    "AC3 2.0 Pro-Logic II Emulation+THX",   // 31
    "DTS Neo:6 Cinema 6",           // 32
    "DTS Neo:6 Cinema 5",           // 33
    "DTS Neo:6 Music",              // 34
    "DTS Neo:6 Cinema 6",           // 35
    "DTS Neo:6 Cinema 5",           // 36
    "AC3 2.0 Classical",            // 37
    "AC3 2.0 General Admission",    // 38
    "AC3 2.0 Front Row",            // 39
    "AC3 2.0 On Stage",             // 40
    "AC3 2.0 Enhanced Stereo",      // 41
    "AC3 2.0 Orchestra",            // 42
    "AC3 2.0 Mezzanine",            // 43
    "AC3 2.0 Full Range+Sub",       // 44
    "AC3 2.0 Monophonic",           // 45
    "AC3 2.0 Preamp",               // 46
    // dolby ac-3 5.1 modes
    "AC3 5.1",                      // 47
    "AC3 5.1+THX",                  // 48
    "AC3 5.1+THX EX",               // 49
    // DTS modes
    "DTS 5.1 Movie",                // 50
    "DTS 5.1 Matrix",               // 51
    "DTS 5.1 Movie+THX",            // 52
    "DTS 5.1 Music",                // 53
    "DTS 5.1 Matrix+THX",           // 54
    "DTS 6.1 Discrete",             // 55
    "DTS 6.1 Discrete+THX",         // 56
    "DTS 5.1 Force"                 // 57
};

template< typename T >
static inline size_t sizeOfArray (T &x)
{
    return sizeof (T) / sizeof (x[0]);
}
#define endOfArray(x) ((x)+sizeOfArray(x))

static bool cStringMatches (const char *lhs, const char *rhs)
{
    return (std::strcmp (lhs, rhs) == 0);
}

static inline const char *findCodeForPropertyValue (
        const char * const *first, const char * const *last, 
        const char *propertyValue)
{
    const char * const *ptr = std::find_if (first, last,
            std::bind2nd (std::ptr_fun (cStringMatches), propertyValue));
    if (ptr >= last)
        return NULL;
    return *ptr + std::strlen (*ptr) + 1;
}

static inline const char *displayValueForIndex (
        const char * const *first, const char * const *last,
        int index)
{
    const char * const *result = first+index;
    if (index < 0 || result >= last)
        return kInvalid;
    return *result;
}

KrellModule::KrellModule (HANDLE dllInstance, const DeviceCallBackInfo *cbi,
        const std::string &version, const std::string &name) :
    DeviceModule (dllInstance, cbi, version, name)
{
}

KrellModule::~KrellModule ()
{
}

const char *KrellModule::compression (int status) const
{
    return displayValueForIndex (kCompression, endOfArray (kCompression), 
            status);
}

const char *KrellModule::compressionCode (const char *propertyValue) const
{
    return findCodeForPropertyValue (kCompression, endOfArray (kCompression),
            propertyValue);
}

const char *KrellModule::device (int status, int deviceNumber) const
{
    const char *result = kInvalid;
    switch (deviceNumber)
    {
    case kDevice_Showcase:
        result = displayValueForIndex (
                kDevices, endOfArray (kDevices), status);
        break;
    case kDevice_HTS71:
        result = displayValueForIndex (
                kMainDevices, endOfArray (kMainDevices), status);
        break;
    }
    
    return result;
}

const char *KrellModule::deviceCode (const char *propertyValue, Zone zone) 
        const
{
    switch (zone)
    {
    case kZone_Current:
        return findCodeForPropertyValue (
                kDevices, endOfArray (kDevices), propertyValue);
    case kZone_Main:
        return findCodeForPropertyValue (
                kMainDevices, endOfArray (kMainDevices), propertyValue);
    
    case kZone_Secondary:
        return findCodeForPropertyValue (
                kZone2Devices, endOfArray (kZone2Devices), propertyValue); 
    }
    return NULL; // hopefully unreached
}

const char *KrellModule::directAudio (int status, int deviceNumber) const
{
    const char *result = kInvalid;
    switch (deviceNumber)
    {
    case kDevice_Showcase:
        result = displayValueForIndex (
                kDirectShowcaseAudio, endOfArray (kDirectShowcaseAudio), 
                status);
        break;
    case kDevice_HTS71:
        result = displayValueForIndex (
                kDirectHTS71Audio, endOfArray (kDirectHTS71Audio), 
                status);
        break;
    }
    return result;
}

const char *KrellModule::directAudioCode (const char *propertyValue, 
        Zone zone) const
{
    switch (zone)
    {
        case kZone_Current:
            return findCodeForPropertyValue (kDirectShowcaseAudio,
                    endOfArray (kDirectShowcaseAudio), propertyValue);
        case kZone_Main:
            return findCodeForPropertyValue (kDirectMainAudio,
                    endOfArray (kDirectMainAudio), propertyValue);
        case kZone_Secondary:
            return findCodeForPropertyValue (kDirectZone2Audio,
                    endOfArray (kDirectZone2Audio), propertyValue);
    }
    return NULL; // hopefully unreached
}

const char *KrellModule::directVideo (int status) const
{
    return displayValueForIndex (kDirectVideo, endOfArray (kDirectVideo),
            status);
}

const char *KrellModule::directVideoCode (const char *propertyValue) const
{
    return findCodeForPropertyValue (kDirectVideo, endOfArray (kDirectVideo),
            propertyValue);
}

const char *KrellModule::audioDecode (int status) const
{
    return displayValueForIndex (kAudioDecode, endOfArray (kAudioDecode),
            status);
}

const char *KrellModule::audioDecodeCode (const char *propertyValue) const
{
    return findCodeForPropertyValue (kAudioDecode, endOfArray (kAudioDecode),
            propertyValue);
}

const char *KrellModule::buttonCode (const char *propertyValue) const
{
    return findCodeForPropertyValue (kButtons, endOfArray (kButtons),
            propertyValue);
}

const char *KrellModule::currentAudioDecode (int status) const
{
    return displayValueForIndex (kCurrentAudioDecode, 
            endOfArray (kCurrentAudioDecode), status);
}

bool KrellModule::initializeModule (void)
{
    // XXX string resources
    
    addDevice (kDevice_Showcase, createDevice, 
            "Krell Showcase Hardware Device",
            dtHardware,
            kKrellSignature_Command,
            dfNoFlags, //dfHelpSupported
"This device interfaces with the Krell Showcase home theater processor",
            "Krell Showcase setup complete");
            
    addDevice (kDevice_HTS71, createDevice,
            "Krell HTS71 Hardware Device",
            dtHardware,
            kKrellSignature_Command,
            dfNoFlags, //dfHelpSupported
"This device interfaces with the Krell HTS71 home theater processor",
            "Krell HTS71 setup complete");
            
    return true;
}

bool KrellModule::registerProperties (int deviceNumber)
{
    if (deviceNumber < 0 || deviceNumber >= (int)m_devices.size ())
        return false;

    static const char *kOnAndOff [] = { "On", "Off" };
    static const char **kNoValue = NULL;

    // since exactly one property can be default, that will be the first one
    
    const struct KrellProperty properties[] = {
        { -1, 
          kKrellProperty_Power, "Off", ioInputAndOutput, ptAlpha, 
                kOnAndOff, endOfArray (kOnAndOff) },
        
        { kDevice_HTS71,
          kKrellProperty_Zone2Power, "Off", ioInputAndOutput, ptAlpha,
                kOnAndOff, endOfArray (kOnAndOff) },
        
        { -1,
          kKrellProperty_DeviceSelect, "", ioInputAndOutput, ptAlpha,
                kDevices, endOfArray (kDevices) },
                
        { kDevice_HTS71,
          kKrellProperty_MainDeviceSelect, "", ioInputAndOutput, ptAlpha,
                kMainDevices, endOfArray (kMainDevices) },
                
        { kDevice_HTS71,
          kKrellProperty_Zone2DeviceSelect, "", ioInputAndOutput, ptAlpha,
                kZone2Devices, endOfArray (kZone2Devices) },
                
        { -1,
          kKrellProperty_VolumeLevel, "0", ioInputAndOutput, ptNumeric,
                kNoValue, kNoValue },
                
        { kDevice_HTS71,
          kKrellProperty_Zone2VolumeLevel, "0", ioInputAndOutput, ptNumeric,
                kNoValue, kNoValue },
                
        { -1,
          kKrellProperty_CenterTrim, "0", ioInputAndOutput, ptNumeric,
                kNoValue, kNoValue },
                
        { -1,
          kKrellProperty_SurroundTrim, "0", ioInputAndOutput, ptNumeric,
                kNoValue, kNoValue },
                
        { -1,
          kKrellProperty_BackTrim, "0", ioInputAndOutput, ptNumeric,
                kNoValue, kNoValue },
                
        { -1,
          kKrellProperty_SubwooferTrim, "0", ioInputAndOutput, ptNumeric,
                kNoValue, kNoValue },
                
        { -1,
          kKrellProperty_Balance, "0", ioInputAndOutput, ptNumeric,
                kNoValue, kNoValue },
        
        // XXX documentation omits any mention of main zone balance?
                
        { kDevice_HTS71,
          kKrellProperty_Zone2Balance, "0", ioInputAndOutput, ptNumeric,
                kNoValue, kNoValue },
                
        { -1,
          kKrellProperty_Compression, *(endOfArray(kCompression)-1), 
                ioInputAndOutput, ptAlpha,
                kCompression, endOfArray (kCompression) },
        
        // there are additional values in the array to satisfy both models
        { -1,
          kKrellProperty_DirectAudio, "", ioInputAndOutput, ptAlpha,
                kDirectShowcaseAudio, endOfArray (kDirectShowcaseAudio) },
                
        { kDevice_HTS71,
          kKrellProperty_MainDirectAudio, "", ioInputAndOutput, ptAlpha,
                kDirectMainAudio, endOfArray (kDirectMainAudio) },
                
        { kDevice_HTS71,
          kKrellProperty_Zone2DirectAudio, "", ioInputAndOutput, ptAlpha,
                kDirectZone2Audio, endOfArray (kDirectZone2Audio) },
                
        { -1,
          kKrellProperty_DirectVideo, "", ioInputAndOutput, ptAlpha,
                kDirectVideo, endOfArray (kDirectVideo) },
                
        { -1,
          kKrellProperty_RoomEQ, "", ioInputAndOutput, ptNumeric,
                kNoValue, kNoValue },
                
        { -1,
          kKrellProperty_Trigger1, "", ioInputAndOutput, ptAlpha,
                kOnAndOff, endOfArray (kOnAndOff) },
                
        { -1,
          kKrellProperty_Trigger2, "", ioInputAndOutput, ptAlpha,
                kOnAndOff, endOfArray (kOnAndOff) },

        { -1,
          kKrellProperty_Trigger3, "", ioInputAndOutput, ptAlpha,
                kOnAndOff, endOfArray (kOnAndOff) },

        { -1,
          kKrellProperty_Trigger4, "", ioInputAndOutput, ptAlpha,
                kOnAndOff, endOfArray (kOnAndOff) },

        { -1,
          kKrellProperty_AudioDecode, kAudioDecode[0], ioOutputOnly, ptAlpha,
                kAudioDecode, endOfArray (kAudioDecode) },
                
        { -1,
          kKrellProperty_Muting, kOnAndOff[1], ioInputAndOutput, ptAlpha,
                kOnAndOff, endOfArray (kOnAndOff) },
                
        // again, there doesn't seem to be a way to directly toggle main mute                
                
        { kDevice_HTS71,
          kKrellProperty_Zone2Muting, kOnAndOff[1], ioInputAndOutput, ptAlpha,
                kOnAndOff, endOfArray (kOnAndOff) },
                
        { -1,
          kKrellProperty_Button, "", ioOutputOnly, ptAlpha,
                kButtons, endOfArray (kButtons) },
                
        { -1,
          kKrellProperty_CurrentAudioDecode, "", ioInputOnly, ptAlpha,
                kCurrentAudioDecode, endOfArray (kCurrentAudioDecode) },
                
        { -1,
          kKrellProperty_InputTrigger1, "", ioInputOnly, ptAlpha,
                kOnAndOff, endOfArray (kOnAndOff) },
                
        { -1,
          kKrellProperty_Mode1, "", ioInputOnly, ptAlpha,
                kOnAndOff, endOfArray (kOnAndOff) },
                
        { kDevice_HTS71,
          kKrellProperty_CurrentZone, "", ioInputOnly, ptAlpha,
                kNoValue, kNoValue }
    };
    const struct KrellProperty *p;
    for (p = properties; p != endOfArray (properties); ++p)
        if (p->deviceNumber == -1 || p->deviceNumber == deviceNumber)
            createAndRegisterProperty (deviceNumber, p->name, p->name, 
                p == &properties[0] ? true : false, p->initialValue,
                p->ioType, true, true, false, false, p->type,
                p->firstValue, p->lastValue);

    return true;
}

CDevice *KrellModule::createDevice (intptr_t data, DEVICE_HANDLE handle,
        const DeviceModule *me, bool isEnabled, const std::string &name,
        const std::string &description)
{
    return new CKrellHardwareDevice (handle, me, isEnabled, name, description, data);    
}
