// ====================================================================
// Original file Copyright (c) 2002, 2004 Meedio, LLC.  All rights reserved.
//
// File          : Device.cpp
// Origin Date   : 09/7/2002
// Purpose       : This file contains the implementation for the CDevice
//                   helper class.  This class aids in the development of
//                   creating Device Plugin Instances.
// ====================================================================

#include "stdafx.h"
#include "Device.h"
#include "DeviceModule.h"
#include "Property.h"

#include <algorithm>
#include <functional>

// ====================================================================
// Method Name : CDevice
// Description : Constructor for CDevice instance helper class
// Parameters  : hDevice - Device Handle.  Passed on HBDeviceInit()
//               pCallBackInfo - Callback info to store.  Passed on HBDeviceModuleInit()
//               bEnabled - Initial enabled state of instance.  Passed on HBDeviceInit()
//               szDeviceName - Name of Device specified in ModuleInfo.
//               szDeiceDescription - User assigned description of Device.
// ====================================================================
CDevice::CDevice (DEVICE_HANDLE device,
                  const DeviceModule *module,
                  bool isEnabled,
                  const std::string &deviceName,
                  const std::string &deviceDescription) :
    m_device (device),
    m_module (module),
    m_cbi (module->callBackInfo ()),
    m_enabled (isEnabled),
    m_deviceName (deviceName),
    m_deviceDescription (deviceDescription)
{
}

// ====================================================================
// Method Name : ~CDevice
// Description : CDevice destructor.
// ====================================================================
CDevice::~CDevice()
{
    propertymap_type::iterator i;
    for (i = m_properties.begin (); i != m_properties.end (); ++i)
        delete i->second;
}

// default virtual methods
bool CDevice::DeviceInit (void) 
{ 
    return false; 
}

bool CDevice::DeviceRun (HANDLE, HANDLE)
{
    return false;
}

void CDevice::ButtonPressNotification (int)
{
}

bool CDevice::SubscriptionNotification (const std::string &listName,
        const std::string &szFilter, const CDataPack *data)
{
    return true;
}

bool CDevice::InitDynamicProperty (PROPERTY_HANDLE)
{
    return false;
}

bool CDevice::SetDynamicProperty (PROPERTY_HANDLE, const std::string &, int)
{
    return false;
}

bool CDevice::DeleteDynamicProperty (PROPERTY_HANDLE, int)
{
    return false;
}

bool CDevice::MoveItemDownDynamicProperty (PROPERTY_HANDLE, int)
{
    return false;
}

bool CDevice::MoveItemUpDynamicProperty (PROPERTY_HANDLE, int)
{
    return false;
}

bool CDevice::GetEnabled (void) const
{
    return m_enabled;
}

void CDevice::SetEnabled (bool isEnabled)
{
    m_enabled = isEnabled;
}


// ====================================================================
// Method Name : CheckForServerVersion
// Description : This method will check to see if the server version is at
//                or above the version values passed.
// Parameters  : nMajor - Integer that specifies the required major version number
//             : nMinor - Integer that specifies the required minor version number
//             : nSubMinor - Integer that specifies the required subminor version number.
// Returns     : TRUE if version is >= values passed.  FALSE if version is < values passed.
// ====================================================================
bool CDevice::CheckForServerVersion( int nMajor, int nMinor, int nSubMinor ) const
{
   int   nMaj = 0;
   int   nMin = 0;
   int   nSubMin = 0;

   if ((!m_cbi) ||
       (!m_cbi->m_szHBVersion))
   {
      return( false );
   }

   if (3 != sscanf( m_cbi->m_szHBVersion, "%d.%d.%d", &nMaj, &nMin, &nSubMin ))
   {
      if (2 != sscanf( m_cbi->m_szHBVersion, "%d.%d", &nMaj, &nMin ))
      {
         return( false );
      }
   }

   //
   // Test the values.
   if (nMaj > nMajor)
   {
      return( true );
   }

   if ((nMaj == nMajor) && (nMin > nMinor))
   {
      return( true );
   }

   if ((nMaj == nMajor) && (nMin == nMinor) && (nSubMin >= nSubMinor))
   {
      return( true );
   }

   return( false );
}

// ====================================================================
// Method Name : GetDeviceID
// Description : This method will return the unique device instance ID.
// Returns     : Device instance ID, or 0 for error.
// ====================================================================
int CDevice::GetDeviceID (void) const
{
   if (m_cbi)
   {
      return( m_cbi->pHB_GetDeviceID( m_device ) );
   }

   return( 0 );
}

// ====================================================================
// Method Name : GetNextPropertyValueOnChangeRequestQueue
// Description : This method will retreive and remove the next property
//                value that has changed from the change queue.
// Returns     : Pointer to a CPropertyValue object that indicates the NEW
//                value that the user wants to change to.
//                Returns NULL if there are no change values on the queue.
// ====================================================================
CPropertyValue* CDevice::GetNextPropertyValueOnChangeRequestQueue()
{
   //
   // Simply pass the call to the CPropertyValue static method.
   return( CPropertyValue::GetNextPropertyValueOnChangeRequestQueue( this ) );
}

// ====================================================================
// Method Name : CreateDeviceProperty
// Description : This function will create a new property.
// Returns     : Returns TRUE if successful, or FALSE if not successful.
// ====================================================================
bool CDevice::CreateDeviceProperty( const std::string &propertyName,
                                    const std::string &propertyDescription,
                                    ePropertyType type )
{
    if (!m_cbi || !m_cbi->pHB_CreateDeviceProperty (
            propertyName.c_str (),
            propertyDescription.c_str (), 
            type))
        return false;
    
    return true;
}

// ====================================================================
// Method Name : CreateDevicePropertyMapping
// Description : Creates a Device Property mapping that does not exist in
//                the Stock/Default device definition.  This method only
//                needs to be called on special circumstances. It is
//                typically NOT needed, since all registered properties
//                are automatically mapped to a new device.  It is only
//                needed when the specific device instance needs specific
//                properties that are not shared with all other device
//                instances of the same type.
// Returns     : TRUE if the mapping was successful.  FALSE if unsuccessful.
//                If unsuccessful, it may be due to the fact that the
//                Property Value does not exist.
// ====================================================================
bool CDevice::CreateDevicePropertyMapping( const std::string &propertyName,
                                           const std::string &propertyDescription,
                                           const std::string &initialValue,
                                           eIOType iotype,
                                           bool isDefault,
                                           bool isPersistent,
                                           bool allowSameValueChange,
                                           bool trace,
                                           bool isDynamic)
{
    if (!m_cbi || !m_cbi->pHB_CreateDevicePropertyMapping (
            m_device,
            propertyName.c_str (),
            propertyDescription.c_str (),
            initialValue.c_str (),
            iotype,
            isDefault ? TRUE : FALSE,
            isPersistent ? TRUE : FALSE,
            allowSameValueChange ? TRUE : FALSE,
            trace ? TRUE : FALSE,
            isDynamic ? TRUE : FALSE))
        return false;
    
    return true;
}

// ====================================================================
// Method Name : GetPropertyByName
// Description : Returns a CProperty object based on a property name.
//                This method will either return a local copy of a CProperty
//                or will create a new CProperty from data queried from the
//                server if the local copy does not exist.
// Parameters  : szPropertyName - Name of the property to return CProperty object.
// Returns     : CProperty object if szPropertyName is a property of the
//                owning device.  Otherwise it returns NULL.
// ====================================================================
CProperty* CDevice::GetPropertyByName (const std::string &propertyName) const
{
    propertymap_type::const_iterator i;
    for (i = m_properties.begin (); i != m_properties.end (); ++i)
        if (i->second->GetPropertyName () == propertyName)
            return i->second;

    CProperty *property = new CProperty (this, propertyName);
    if (!property->GetPropertyHandle ())
    {
        delete property;
        return NULL;
    }
    return property;
}

// ====================================================================
// Method Name : GetPropertyByDescription
// Description : Returns a CProperty object based on a property description.
//                This method will either return a local copy of a CProperty
//                or will create a new CProperty from data queried from the
//                server if the local copy does not exist.
// Parameters  : szPropertyDescription - User given description of the property
//                to return CProperty object.
// Returns     : CProperty object if szPropertyName is a property of the
//                owning device.  Otherwise it returns NULL.
// ====================================================================
CProperty* CDevice::GetPropertyByDescription ( 
        const std::string &propertyDescription) const
{
    propertymap_type::const_iterator i;
    for (i = m_properties.begin (); i != m_properties.end (); ++i)
        if (i->second->GetPropertyDescrption () == propertyDescription)
            return i->second;

    PROPERTY_HANDLE   handle;
    const char*       szPropertyName;
    if (!m_cbi->pHB_GetPropertyInfoByDescription (m_device, 
            propertyDescription.c_str (), &handle, &szPropertyName))
        return NULL;

    CProperty *property = new CProperty (this, std::string (szPropertyName),
            propertyDescription, handle);
    if (!property->GetPropertyHandle ())
    {
        delete property;
        return NULL;
    }
    m_properties[property->GetPropertyHandle ()] = property;
    return property;
}

// ====================================================================
// Method Name : CreatePropertyByName
// Description : This methods will create a CProperty object from the supplied
//                property name, description (,and optinally handle).
// Parameters  : szPropertyName - Name of the property to create.
//               szPropertyDescription - User given description of the property.
//               hProperty - Optional property handle.
// Returns     : Pointer to CProperty object if name and description match a
//                property managed by the device.  Else it returns NULL.
// ====================================================================
CProperty* CDevice::CreatePropertyByName( 
    const std::string &propertyName, 
    const std::string &propertyDescription, 
    PROPERTY_HANDLE handle)
{
    //
    // Create the new property
    CProperty *property;
    
    property = new CProperty (this, propertyName, propertyDescription, handle);
        
    //
    // Make sure the new handle is valid.  If it is invalid, there was a propble finding the property.
    if (!property->GetPropertyHandle())
    {
        delete property;
        return NULL;
    }

    m_properties[property->GetPropertyHandle()] = property;
    return property;
}

CProperty *CDevice::CreatePropertyByName (const std::string &propertyName)
{
    CProperty *property = new CProperty (this, propertyName);
    if (!property->GetPropertyHandle ())
    {
        delete property;
        return NULL;
    }
    
    m_properties[property->GetPropertyHandle ()] = property;
    return property;
}

// ====================================================================
// Method Name : GetPropertyFromHandle
// Description : This method will return a CProperty object based on the
//                supplied property handle.  The method will first try and
//                return a local cached copy of the property.  If the property
//                does not exist in the cache, it will create a new CProperty
//                object and add it to the cache.
// Parameters  : hProperty - Handle of the property to return.
// Returns     : Pointer to a CProperty object if the handle is valid.
//                If handle is invalid, returns NULL.
// ====================================================================
CProperty* CDevice::GetPropertyFromHandle (PROPERTY_HANDLE handle) const
{
    propertymap_type::const_iterator i = m_properties.find (handle);
    if (i != m_properties.end ())
        return i->second;

    CProperty *property = new CProperty (this, handle);
    if (!property->GetPropertyHandle ())
        return NULL;
    
    m_properties[handle] = property;
    return property;
}

// ====================================================================
// Method Name : SendInterfacePackToInterface
// Description : This method will send an InterfaceArgumentPack to the
//                Devices configured interface.
// Parameters  : InterfaceArgumentPack* - Pointer to InterfaceArgumentPack
//                data that include the interface parameters.
// Returns     : Possible return values from the interface are:
//                hrcFailureRetry - It failed, but retry.
//                hrcFailureAbort - It failed, don't retry.
//                hrcSuccess - It succeeded.
// ====================================================================
HARDWARE_RC CDevice::SendInterfacePackToInterface( 
        const InterfaceArgumentPack* pPack )
{
   return( m_cbi->pHB_SendInterfacePackToInterface( m_device, pPack ) );
}

// ====================================================================
// Method Name : QueryHardwareInstance
// Description : This method will send a general query to the hardware interface
//                associated with the Device.
// Parameters  : szQuery - Null terminated string that contains the query.
//               pResponse - Pointer to query response buffer.  The specifics of the data type and format
//                is well-known between the device and the interface.
// Returns     : TRUE if the interface successfully processed the query.
//               FALSE if the interface did not successfully process the query.
// ====================================================================
bool CDevice::QueryHardwareInstance (const std::string &query, 
        void *response) const
{
    if (m_cbi && m_cbi->pHB_QueryHardwareInstance (m_device, query.c_str (),
            response))
        return true;
    return false;
}

// ====================================================================
// Method Name : TraceMessage
// Description : This method will trace a message using the servers
//                tracing services.
// Parameters  : ttType - Trace type.  Valid values are:
//                   ttError - Trace to error log.
//                   ttWarning - Trace as warning message.
//                   ttDebug - Trace as debug message.
// ====================================================================
void CDevice::TraceMessage( TRC_TYPE ttType, const char* szMessage, ... ) const
{
   char  szFormattedMessage[ 512 ];

   //
   // Format the message
   va_list args;
   va_start( args, szMessage );
   wvsprintf( szFormattedMessage, szMessage, args );

   //
   // Send it to the server.
   m_cbi->pHB_TraceDeviceMessage( m_device, ttType, szFormattedMessage );
}

// ====================================================================
// Method Name : SubscribeToNotificationList
// Description : This method will subscribe the Device to a "well known"
//                Notification list.  Notification Lists enable Hardware
//                Interfaces to send notification data to interested Devices
//                when changes are detected by the Hardware.
// Parameters  : szListName - Well Known list name.
//                Some existing list names are:
//                "Display Setting Change"
//                "X10 Reception"
//                "X10 Reception (ALL)"
//                "Input State Change"
//                "X10 Global Reception"
//                "X10 Global Reception (ALL)"
//                "IR Response Received"
//                "Ring Information Received"
//                "Caller ID Information Received"
// szFilter : Additional information to help Devices filter their subscription.  For example, if an X10 device only
//             required notifications when a particular House Code or Unit Code changed, it would subscribe with a
//             filter value of "H=h;U=u" where h and u represent house code and unit code.  If there is no filter to
//             specify, NULL can be passed.
// ====================================================================
void CDevice::SubscribeToNotificationList (const std::string &listName, 
        const std::string &filter )
{
    if (m_cbi)
        m_cbi->pHB_SubscribeToNotificationList (m_device, listName.c_str (), 
            filter.empty () ? NULL : filter.c_str ());
}

// ====================================================================
// Method Name : UnSubscribeFromNotificationList
// Description : This method will remove the device from the specified
//                subscription list (and optinally, filter).
// Parameters  : szListName - This is the same list name that would have
//                been supplied earlier with SubscribeToNotificationList()
//               szFilter - This is the same filter that would have been
//                supplied earlier with SubscribeToNotificationList().
// ====================================================================
void CDevice::UnSubscribeFromNotificationList (const std::string &listName,
        const std::string &filter)
{
    if (m_cbi)
        m_cbi->pHB_UnSubscribeFromNotificationList (m_device, 
                listName.c_str (), filter.c_str ());
}

// ====================================================================
// Method Name : GetDeviceFromDescription
// Description : This method will return a CDevice object from a given
//                Device description.
// Parameters  : szDeviceDescription - User assigned device name.
// Returns     : Pointer to a CDevice object if the Device Description
//                is valid.  NULL if the Description is not valid.
// ====================================================================
CDevice* CDevice::GetDeviceFromDescription (const std::string &description) const
{
    DEVICE_HANDLE  device;
    BOOL           isEnabled;
    const char*    szDeviceName;

    if (m_cbi->pHB_GetDeviceInfoFromDescription (description.c_str (),
            &device, &isEnabled, &szDeviceName))
        return new CDevice (device, m_module, isEnabled ? true : false, 
            std::string (szDeviceName), description);
    
    return NULL;
}

// ====================================================================
// Method Name : GetModeState
// Description : This method will return the state of the specified Mode.
// Parameters  : szModeName - Name of the mode whose state is requested.
// Returns     : "Active" if the mode is Active.
//               "Inactive" if the mode is Inactive.
//               NULL if the mode name is not valid.
// ====================================================================
const char *CDevice::GetModeState( const std::string &modeName) const
{
    return m_cbi->pHB_GetModeStateByName (modeName.c_str ());
}

// ====================================================================
// Method Name : SetModeState
// Description : This method will change the state of the specified Mode.
// Parameters  : szModeName - Name of the Mode to change state.
//               bActive - TRUE to set state as Active.
//                         FALSE to set state as Inactive.
// Returns     : TRUE if state changed.  FALSE if szModeName is not valid.
// ====================================================================
bool CDevice::SetModeState (const std::string &modeName, bool active)
{
    if (m_cbi->pHB_SetModeStateByName (modeName.c_str (), 
            active ? TRUE : FALSE))
        return true;
    return false;
}

// ====================================================================
// Method Name : ExecuteTask
// Description : This method will execute the task specified.
// Parameters  : szTaskName - Name of the task to execute.
// Returns     : TRUE if the task executed.  FALSE if the task name
//                is invalid, OR if the task did not execute due to
//                it being inactive or conditions not met.
// ====================================================================
bool CDevice::ExecuteTask (const std::string &taskName )
{
    if (m_cbi && m_cbi->pHB_ExecuteTaskByName (taskName.c_str ()))
        return true;
    return false;
}

// ====================================================================
// Method Name : IsTaskRunning
// Description : This method will check the running state of a task.
// Parameters  : szTaskName - Name of the Task to check.
// Returns     : TRUE if the task is running.  FALSE if the task is
//                not running or if the task name is not valid.
// ====================================================================
bool CDevice::IsTaskRunning (const std::string &taskName ) const
{
    if (m_cbi && m_cbi->pHB_IsTaskRunning (taskName.c_str ()))
        return true;
    return false;
}

// ====================================================================
// Method Name : StopTask
// Description : This method will terminate the specified Task.
// Parameters  : szTaskName - Name of the task to  terminate.
// ====================================================================
void CDevice::StopTask (const std::string &taskName)
{
    if (m_cbi)
        m_cbi->pHB_StopTaskByName (taskName.c_str ());
}

// ====================================================================
// Method Name : ControlRemote
// Description : This method control Software Remotes
// Parameters  : See DeviceAPI.h for more information.
// ====================================================================
bool CDevice::ControlRemote( eClientControlAction action,
                             const std::string &whichRemotes,
                             const std::string &arg1,
                             const std::string &arg2 )
{
    if (m_cbi && m_cbi->pHB_ControlRemote (action, whichRemotes.c_str (),
            arg1.c_str (), arg2.c_str ()))
        return true;
    return false;
}

// ====================================================================
// Method Name : GetDeviceList
// Description : This method will return the list of system Devices
// Parameters  : pszDeviceList - Buffer for Device list.  Send NULL to get size only.
//               pdwBufferSize - Size of pszDeviceList buffer.
//               szDelimiter - Characters used to delimit Devices returned in buffer.
// ====================================================================
std::string CDevice::GetDeviceList (const char *delimiter) const
{
    std::string result;
    DWORD sizeNeeded = 0;
    if (m_cbi && m_cbi->pHB_GetDeviceList (NULL, &sizeNeeded, delimiter))
    {
        result.resize (sizeNeeded+1, 0);
        if (m_cbi->pHB_GetDeviceList (&(*result.begin ()), &sizeNeeded, delimiter))
            result.resize (std::strlen (result.data ()));
        else
            result.clear ();
    }
    return result;
}

// ====================================================================
// Method Name : GetPropertyListForDevice
// Description : This method will return the list of Properties associated with the specified Device
// Parameters  : pszDeviceDescription - Description of Device to return Properties for
//               pszPropertyList - Buffer for Property list.  Send NULL to get size only.
//               pdwBufferSize - Size of pszPropertyList buffer.
//               szDelimiter - Characters used to delimit Devices returned in buffer.
// ====================================================================
std::string CDevice::GetPropertyListForDevice( 
        const std::string &deviceDescription, const char *delimiter) const
{
    std::string result;
    DWORD sizeNeeded = 0;
    if (m_cbi && m_cbi->pHB_GetPropertyListForDevice (deviceDescription.c_str (),
            NULL, &sizeNeeded, delimiter))
    {
        result.resize (sizeNeeded+1, 0);
        if (m_cbi->pHB_GetPropertyListForDevice (deviceDescription.c_str (),
                &(*result.begin ()), &sizeNeeded, delimiter))
            result.resize (std::strlen (result.data ()));
        else
            result.clear ();
    }
    
    return result;
}

// ====================================================================
// Method Name : GetPropertyValueListForProperty
// Description : This method will return the list of valid Property Values for the specified Property
// Parameters  : pszPropertyDescription - Description of Property to return the value list for.
//               pszValueList - Buffer for Value list.  Send NULL to get size only.
//               pdwBufferSize - Size of pszValueList buffer.
//               szDelimiter - Characters used to delimit Values returned in buffer.
// ====================================================================
std::string CDevice::GetPropertyValueListForProperty( 
        const std::string &propertyDescription, const char *delimiter) const
{
    std::string result;
    DWORD sizeNeeded = 0;
    if (m_cbi && m_cbi->pHB_GetPropertyValueListForProperty (
            propertyDescription.c_str (), NULL, &sizeNeeded, delimiter))
    {
        result.resize (sizeNeeded+1, 0);
        if (m_cbi->pHB_GetPropertyValueListForProperty (
                propertyDescription.c_str (), &(*result.begin ()), &sizeNeeded,
                delimiter))
            result.resize (std::strlen (result.data ()));
        else
            result.clear ();
    }
    
    return result;
}

// ====================================================================
// Method Name : GetTaskList
// Description : This method will return the list Task names configured in the system.
// Parameters  : pszTaskList - Buffer for list of Task names.  Send NULL to get size only.
//               pdwBufferSize - Size of pszTaskList buffer.
//               szDelimiter - Characters used to delimit Values returned in buffer.
// ====================================================================
std::string CDevice::GetTaskList (const char *delimiter) const
{
    std::string result;
    DWORD sizeNeeded = 0;
    if (m_cbi && m_cbi->pHB_GetTaskList (NULL, &sizeNeeded, delimiter))
    {
        result.resize (sizeNeeded+1, 0);
        if (m_cbi->pHB_GetTaskList (&(*result.begin ()), &sizeNeeded, 
                delimiter))
            result.resize (std::strlen (result.data ()));
        else
            result.clear ();
    }
    
    return result;
}

// ====================================================================
// Method Name : GetModeList
// Description : This method will return the list Mode names configured in the system.
// Parameters  : pszModeList - Buffer for list of Mode names.  Send NULL to get size only.
//               pdwBufferSize - Size of pszModeList buffer.
//               szDelimiter - Characters used to delimit Values returned in buffer.
// ====================================================================
std::string CDevice::GetModeList (const char *delimiter) const
{
    std::string result;
    DWORD sizeNeeded = 0;
    if (m_cbi && m_cbi->pHB_GetModeList (NULL, &sizeNeeded, delimiter))
    {
        result.resize (sizeNeeded+1, 0);
        if (m_cbi->pHB_GetModeList (&(*result.begin ()), &sizeNeeded,
                delimiter))
            result.resize (std::strlen (result.data ()));
        else
            result.clear ();
    }
    
    return result;
}

// ====================================================================
// Method Name : ParseTimeSpan
// Description : This method will parse a time span property value as
//                returned by the server into separate elements.
// Parameters  : szValue - The timespan value as returned by the server.
//               &nDays - Reference for the returned number of days in span
//               &nHours - Reference for the returned number of hours in span
//               &nMinutes - Reference for the returned number of minutes in span
//               &nSeconds - Reference for the returned number of seconds in span
//               &nMilliSeconds - Reference for the returned number of milli-seconds in span
// ====================================================================
void CDevice::ParseTimeSpan( const std::string &value, int *days, int *hours, 
        int *minutes, int *seconds, int *milliSeconds )
{
    size_t n;
    
    n = value.find_first_of ("Days=");
    if (n != value.npos)
        *days = atoi (value.substr (n+5).c_str ());
    
    n = value.find_first_of ("Hours=");
    if (n != value.npos)
        *hours = atoi (value.substr (n+5).c_str ());
    
    n = value.find_first_of ("Minutes=");
    if (n != value.npos)
        *minutes = atoi (value.substr (n+8).c_str ());
    
    n = value.find_first_of ("Seconds=");
    if (n != value.npos)
        *seconds = atoi (value.substr (n+8).c_str ());

    n = value.find_first_of ("MilliSeconds=");
    if (n != value.npos)
        *milliSeconds = atoi (value.substr (n+13).c_str ());
}
