// ====================================================================
// Original file Copyright (c) 2002, 2004 Meedio, LLC.  All rights reserved.
//
// File          : Device.h
// Origin Date   : 09/7/2002
// Purpose       : This file contains the definition for the CDevice
//                   helper class.  This class aids in the development of
//                   creating Device Plugin Instances.
// ====================================================================

#ifndef MEEDIO_DEVICE_H
#define MEEDIO_DEVICE_H

//
// Include common Device plugin definitions.
#include "DeviceAPI.h"
#include "Property.h"

#include <map>
#include <string>

//
// Forward declarations
class CDataPack;
class DeviceModule;

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
// CDevice - This class aids in the development of creating
//            Device Plugin Instances.
//
//           See implementation file (Device.cpp) for method descriptions
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
class CDevice
{
public:
    typedef std::map< PROPERTY_HANDLE, CProperty * > propertymap_type;

public:
    CDevice (DEVICE_HANDLE device, const DeviceModule *, 
             bool enabled, const std::string &deviceName, 
             const std::string &deviceDescription );

    virtual ~CDevice();

    inline const DeviceModule *GetDeviceModule (void) const
        { return m_module; }

    //
    // Return the Device callback info structure
    inline const DeviceCallBackInfo* GetCallBackInfo() const
        { return( m_cbi ); }

    //
    // Check for a particular server version level
    bool CheckForServerVersion( int nMajor, int nMinor, int nSubMinor ) const;

    //
    // Return the handle for this device.
    inline DEVICE_HANDLE GetDeviceHandle() const
        { return( m_device ); }

    //
    // Get and set the enabled state of the device
    virtual bool GetEnabled (void) const;
    virtual void SetEnabled (bool);

    //
    // Sent when a button property is pressed in the Device property view from the HouseBot server.
    virtual void ButtonPressNotification (int buttonId);

    //
    // Return the Device name and Description
    inline const std::string &GetDeviceName (void) const
        { return m_deviceName; }
    inline const std::string &GetDeviceDescription (void) const
        { return m_deviceDescription; }

    CPropertyValue *GetNextPropertyValueOnChangeRequestQueue();

    ////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////
    //           See implementation file (Device.cpp) for method descriptions
    ////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////
    CProperty *CreatePropertyByName (const std::string &propertyName, 
            const std::string &propertyDescription, 
            PROPERTY_HANDLE hProperty);

    CProperty *CreatePropertyByName (const std::string &propertyName);

    CProperty *GetPropertyByName (const std::string &propertyName) const;
    CProperty *GetPropertyByDescription (
            const std::string &propertyDescription) const;
    CProperty *GetPropertyFromHandle (PROPERTY_HANDLE) const;

    int GetDeviceID (void) const;

    HARDWARE_RC SendInterfacePackToInterface (
            const InterfaceArgumentPack* pPack);
    bool QueryHardwareInstance (const std::string &query, void *response) const;

    void TraceMessage (TRC_TYPE ttType, const char* szMessage, ... ) const;

    virtual bool DeviceInit (void);
    virtual bool DeviceRun (HANDLE hTerminate, HANDLE hPropertyChangeRequest);

    //
    // Only reqired for devices that need information from hardware interfaces
    virtual bool SubscriptionNotification (const std::string &listName, 
            const std::string &szFilter, const CDataPack* pDataPack );

    //
    // Only required for devices that support dynamic (device managed) properties)
    virtual bool InitDynamicProperty (PROPERTY_HANDLE prop);
    virtual bool SetDynamicProperty (PROPERTY_HANDLE prop, 
            const std::string &value, int index);
    virtual bool DeleteDynamicProperty (PROPERTY_HANDLE prop, int index);
    virtual bool MoveItemDownDynamicProperty (PROPERTY_HANDLE prop, int index);
    virtual bool MoveItemUpDynamicProperty (PROPERTY_HANDLE prop, int nIndex);

    CDevice *GetDeviceFromDescription (const std::string &deviceDescription) const;

    const char *GetModeState (const std::string &modeName) const;
    bool SetModeState (const std::string &modeName, bool active);

    bool ExecuteTask (const std::string &taskName);
    bool IsTaskRunning (const std::string &taskName) const;
    void StopTask (const std::string &taskName);

    bool CreateDeviceProperty (const std::string &propertyName,
            const std::string &propertyDescription, ePropertyType type);

    //
    // Creates a Device Property mapping that does not exist in the Stock/Default device definition.
    bool CreateDevicePropertyMapping (const std::string &propertyName,
            const std::string &propertyDescription,
            const std::string &initialValue,
            eIOType ioType, 
            bool isDefault, 
            bool isPersistent,
            bool allowSameValueChange,
            bool trace,
            bool isDynamic);

    bool ControlRemote (eClientControlAction action, const std::string &remotes,
            const std::string &arg1, const std::string &arg2);
            
    std::string GetDeviceList (const char *delimiter) const;
    
    std::string GetPropertyListForDevice (const std::string &deviceDescription,
            const char *delimiter) const;
            
    std::string GetPropertyValueListForProperty (
            const std::string &propertyDescription, const char *delimiter) const;
            
    std::string GetTaskList (const char *delimiter) const;
    
    std::string GetModeList (const char *delimiter) const;

protected:
    bool                        m_enabled;
    std::string                 m_deviceName;
    std::string                 m_deviceDescription; 
    const DeviceModule          *m_module;
    const DeviceCallBackInfo    *m_cbi;
    DEVICE_HANDLE               m_device;
    mutable propertymap_type    m_properties;

protected:
    void ParseTimeSpan (const std::string &value, int *days, int *hours, 
            int *minutes, int *seconds, int *milliseconds);

    void SubscribeToNotificationList (const std::string &name, 
            const std::string &filter);
    void UnSubscribeFromNotificationList (const std::string &name,
            const std::string &filter);
};

#endif /* MEEDIO_DEVICE_H */
