// ====================================================================
// Copyright (c) 2002, 2004 Meedio, LLC.  All rights reserved.
//
// File          : DeviceAPI.h
// Origin Date   : 09/7/2002
// Purpose       : This file contains the structure definintions and
//                   enumerations for supporting HouseBot Device Plugins
// ====================================================================

// ====================================================================
// Version Information
// Ver      Date         Eng   Description
// ---------------------------------------
// 1.00     09/7/2002    swl   Initial Version
// 1.01     11/18/2002   swl   Added support for Windows Message data types
//                             Added pHB_GetWinMsgArrayAssociatedWithPropertyValue callback definition
//                             Added pHB_CreateDeviceProperty callback definition
//                             Added pHB_CreateDevicePropertyValue callback definition
//                             Added pHB_CreateDevicePropertyMapping callback definition
//                             Added pHB_GetDeviceID callback definition
//                             Added pHB_AddNewPropertyValue callback definition
// 1.02     12/03/2002   swl   Added HBDisplayHelp() export definition.
// 1.03     12/27/2002   swl   Added pHB_GetGenericCommandAssociatedWithValue callback definition
//                             Added GenericCommand structure
// 1.04     02/24/2003   swl   Added 3 new property value type definitions (ptPath, ptFileName, ptPathAndFileName)
// 1.05     02/26/2003   swl    Modified the following methods to pass pointer to
//                               allocated memory instead of using memory returned from HB.
//                               pHB_GetIRCodeArrayAssociatedWithPropertyValue()
//                               pHB_GetWinMsgArrayAssociatedWithPropertyValue()
//                               pHB_GetGenericCommandsAssociatedWithValue()
//                             Added 2 new Device Files:
//                               dfHelpSupported
//                               dfHelpNOTSupported
// 1.06     03/25/2003   swl   Added pHB_ChangeRequestAbandoned callback definition.
// 1.07     11/11/2003   swl   Added pHB_ControlRemote callback definition.
//                             Added pHB_GetDeviceList
//                             Added pHB_GetPropertyListForDevice
//                             Added pHB_GetPropertyValueListForProperty
//                             Added pHB_GetTaskList
//                             Added pHB_GetModeList
// 1.08     12/04/2003   swl   Added HBAboutBox() export definition.
// 2.00     01/27/2004   swl   Added m_szModuleWizardDescription to the DeviceInfo struct.
//                             Added m_szWizardFinalText to the DeviceInfo struct.
//                             Added pHB_RegisterDevicePropertyUsingWizard callback definition.
//                             Removed MAX_PROPERTY_VALUE_LENGTH limit for Property Values
//                             Added pHB_GetAndRemoveNextChangeRequestOnQueue2() callback to handle larger Property Value lengths
//                             Added pHB_RegisterDeviceButton() to register a button callback
//                             Added new optional export HBDeviceButtonPressNotification()
// ====================================================================

#pragma once

#include "../../syCommon/HBTypes.h"
#include "../../syCommon/InterfaceAPI.h"

#define HOUSEBOT_DEVICE_API   __declspec(dllexport)

//
// Device types
enum eDeviceType
{
   dtUnknown,           // Reserved, don't use
   dtHardware,          // Hardware device
   dtSoftware,          // Software device
   dtLastDeviceType     // Reserved, don't use
};

//
// Input / Output types for properties.
// !! These are currenly NOT supported !!
enum eIOType
{
   ioUnknown,           // Reserved, don't use
   ioInputOnly,         // Input ONLY property
   ioOutputOnly,        // Output ONLY property
   ioInputAndOutput,    // Input and output property
   ioLastIO             // Reserved, don't use
};

//
// Valid property types.
enum ePropertyType
{
   ptNumeric,
   ptAlpha,
   ptTime,
   ptTimeSpan,
   ptBoolean,
   ptHouseCode,
   ptUnitCode,
   ptAlphaList,
   ptPath,
   ptFileName,
   ptPathAndFileName,
   ptLastPropertyType,
};

//
// Windows Message Type identifier for WinMsg structure
enum eWinMsgType
{
   wmtUnknown,
   wmtPause,
   wmtKeyStroke,
   wmtSpecialKey,
   wmtLastMessageType
};

//
// Windows Message Key state identifier for WinMsg structure
enum eWinMsgKeyStates
{
   wmksAlt  = 0x1,
   wmksCtl  = 0x2,
   wmksSft  = 0x4,
   wmksPost = 0x8
};

enum eWinMsgMouseEvents
{
   meUnknown,
   meUp,
   meDown,
   meLeft,
   meRight,
   meLeftClick,
   meRightClick,
   meLastMouseEvent
};

//
// Windows Message definition
struct WinMsg
{
   WinMsg() { m_wmtMessageType = m_wmksKeyStates = m_nValue = 0; }
   WORD        m_wmtMessageType;             // Type of Windows Message data contained in structure.  See eWinMsgType.
   WORD        m_wmksKeyStates;              // Key states.  See eWinMsgKeyStates.
   int         m_nValue;                     // Type specific value.
};

//
// Generic Command definition
struct GenericCommand
{
   GenericCommand() { m_szCommand = NULL; }
   const char* m_szCommand;                  // Command string.
};

//
// Device flag bit mask values
enum eDeviceFlags
{
   dfNoFlags                  = 0x00,
   dfWindowsMessageSupport    = 0x01,              // Indicates that the plugin supports Windows Messages.  This causes the server to
                                                   // display the Windows Messages mapping property page when configuring property values.
   //
   // These two flags control help support.  If neither of them are defined, HB determines
   // help support by the presense of the HBDisplayHelp export.
   dfHelpSupported            = 0x02,              // Indicates that the plugin supports on-line help.
   dfHelpNOTSupported         = 0x04,              // Indicates that the plugin does NOT support on-line help.

   dfUseMeedioAboutBox      = 0x08,              // Used for Meedio Plugins only.  Displays default Meedio about box.
};


//
// DeviceInfo structure used to return Device specific
// information to the server.
struct DeviceInfo
{
   int            m_nDeviceInfoSize;            // Intialized at runtime with the sizeof() the structure
   const char*    m_szDeviceName;               // Name of the Device
   eDeviceType    m_eDeviceType;                // Type (hardware or software) of the Device
   const char*    m_szInterfaceSignatures;      // For hardware devices, list of signagures requried to support the device.  Concatenate interface signagures with "|"
   eDeviceFlags   m_dfFlags;                    // Additional flag values.
   const char*    m_szModuleWizardDescription;  // Description of the plugin.  Used in Wizard.
   const char*    m_szWizardFinalText;          // Text displayed at the end of the Wizard process.
};


//
// Enum for pHB_ControlRemote callback
enum eClientControlAction
{
   ccaOpenPanel,
   ccaClosePanel,
   ccaExecuteApp,
   ccaMinimize,
   ccaRestore,
   ccaBlankScreen,
   ccaControlFocus,

   ccaUnknown
};

//
// Enum for pHB_ControlRemote callback (pArg1 if dealing with control focus)
enum eFocusControl
{
   fcFocusUp,
   fcFocusDown,
   fcFocusLeft,
   fcFocusRight,
   fcFocusSetValue,
   fcFocusEnter,
};



//
// Forward class declarations
class CDataPack;


//
// Internal Note: This structure must match the struct defined in ServerInterface.h and DeviceModule.h
struct DeviceCallBackInfo
{
   //
   // Current version of the HouseBot Server.
   char*                   m_szHBVersion;

   //
   // Module handle assigned by server
   DEVICE_MODULE_HANDLE    m_hModuleHandle;

   //
   // Full path and filename to the plugin DLL
   const char*             m_szDLLPathAndFileName;

   //
   // This function will trace a message on the servers trace log.
   void (*pHB_TraceDeviceMessage)( DEVICE_HANDLE hDevice, TRC_TYPE trcType, const char* szMessage );
   // Parameters:
   // trcType : See values for TRC_TYPE above for possible values.
   // szMessage : Message to trace.

   //
   // This function will cause the server to update all device properties that are dependent
   // on sunrise and sunset times.
   void (*pHB_ResetSunRiseAndSunSetTimes)();

   //
   // This function will return a property value from a given property handle.
   BOOL (*pHB_GetDevicePropertyValueFromPropertyHandle)( PROPERTY_HANDLE hProperty,
                                                         PROPERTY_VALUE_HANDLE *phPropertyValue,
                                                         const char** szPropertyValue,
                                                         const char** szConfigValue );
   // Parameters:
   // hProperty : Handle of Property to retrieve property value handle.
   // phPropertyValue - Pointer to return for property value handle.
   // szPropertyValue - Pointer to return for proprety value string.  NULL if not interested.
   // szConfigValue - Poitner to return for property value config string.  NULL if not interested.

   //
   // This function will return the property value name and config value of specified property value.
   BOOL (*pHB_GetDevicePropertyValueFromHandle)( PROPERTY_VALUE_HANDLE hPropertyValue,
                                                 const char** szPropertyValue,
                                                 const char** szConfigValue );
   // Parameters:
   // hPropertyValue - Property Value handle of property value to lookup.
   // szPropertyValue - Return buffer for property value string.
   // szConfigValue - Return buffer for property value config string.

   //
   // This function will register a device property.  To have HouseBot prompt the user for a Property value
   // when the Device is being created, use pHB_RegisterDevicePropertyUsingWizard() instead of pHB_RegisterDeviceProperty().
   BOOL (*pHB_RegisterDeviceProperty)( DEVICE_MODULE_HANDLE hDeviceModule,
                                       int nDeviceNumber,
                                       const char* szPropertyName,
                                       const char* szPropertyDescription,
                                       BOOL bDefault,
                                       const char* szInitialValue,
                                       eIOType ioType,
                                       BOOL bPersistent,
                                       BOOL bAllowSameValueChange,
                                       BOOL bTraceToDeviceLog,
                                       BOOL bDynamic );
   // Parameters:
   // hDeviceModule - Handle to device module to register property.
   // nDeviceNumber - Device number (ordinal) to register property for.  This matches nDeviceNumber ordinal returned in HBGetDeviceInfo().
   // szPropertyName - Name of property.  Must match pre-defined server property names.
   // szPropertyDescription - Default property description.  Can be changed by user.
   // bDefault - TRUE if this is the default property for the device.  Otherwise it is FALSE.  Each device must have ONLY ONE default property.
   // szInitialValue - Initial value of the property.
   // ioType - Not currently used, but can/should be set correctly.
   // bPersistent - TRUE if this value is to be saved to the DB on each change.  FALSE if the value is not to be saved.
   // bAllowSameValueChange - TRUE if this value is allowed to be changed to the same value.  FALSE if it can only be changed to a new value.
   // bTraceToDeviceLog - TRUE if this value should trace its property changes to the device log.  FALSE if it should not.
   // bDynamic - TRUE if this is a dynamic property.  NOTE:  Typically this is FALSE.

   //
   // This function will return the property name and description from the specified handle.
   BOOL (*pHB_GetPropertyInfoFromHandle)( PROPERTY_HANDLE hProperty, char* szPropertyName, char* szPropertyDescription );
   // Parameters:
   // hProperty - Property handle to retrieve name and description.
   // szPropertyName - Return buffer for the property name.
   // szPropertyDescription - Return buffer for the property description.

   //
   // This function will return a property handle and description from a given property name.
   PROPERTY_HANDLE (*pHB_GetDevicePropertyHandleFromName)( DEVICE_HANDLE hDevice,
                                                           const char* szPropertyName,
                                                           char* szPropertyDescription );
   // Parameters:
   // hDevice : Device Handle
   // szPropertyName : Name of the property to find handle for.
   // szPropertyDescription : Return buffer for the property description

   //
   // This function will create a new property value.
   PROPERTY_VALUE_HANDLE (*pHB_CreatePropertyValue)( PROPERTY_HANDLE hProperty, const char* szNewValue );
   // Parameters:
   // hProperty - Handle to parent/owning property
   // szNewValue - Initial value of new property value.

   //
   // This function will return an array of IR code positions that are associated with the
   // specified property value.  The return value is TRUE if succcessful, or FALSE if not succcessful.
   BOOL (*pHB_GetIRCodeArrayAssociatedWithPropertyValue)( PROPERTY_HANDLE hProperty, PROPERTY_VALUE_HANDLE hPropertyValue, unsigned short* pRetArray, DWORD* pdwBufferSize );
   // Parameters:
   // hPropertyValue : Property handle of owning property.
   // hPropertyValue : Property value needing the IR Code array.
   // pRetArray      : Pointer to buffer to hold array of unsigned short values containing the IR Code positions.  If this parameter is NULL, the function
   //                   will return the size in bytes needed in pdwBufferSize.
   // pdwBufferSize : Pointer to the size of the buffer passed in pRetArray.  Upon return, this value
   //                   will contain the size of pRetArray.

   //
   // This function will add a property value to the device change request queue.
   void (*pHB_ChangeRequest)( PROPERTY_VALUE_HANDLE hPropertyValue );
   // Parameters:
   // hPropertyValue : Handle to property value to place on queue.

   //
   // This function will indicate to the server that the device has successfully changed
   // the property value specified.
   void (*pHB_ChangeRequestCompleted)( PROPERTY_VALUE_HANDLE hPropertyValue );
   // Parameters:
   // hPropertyValue : Property value handle that was successfully changed.

   //
   // This function will indicate to the server that the device has NOT successfully changed
   // the property value specified.
   void (*pHB_ChangeRequestFailed)( PROPERTY_VALUE_HANDLE hPropertyValue );
   // Parameters:
   // hPropertyValue : Property value handle that was NOT successfully changed.

   //
   // This function will return the next property value from the Device property change request queue.
   // It will also remove the change request from the queue.
   // Note:  This callback has been obsoleted by pHB_GetAndRemoveNextChangeRequestOnQueue2().
   BOOL (*pHB_GetAndRemoveNextChangeRequestOnQueue)( DEVICE_HANDLE hDevice,
                                                     PROPERTY_HANDLE *phProperty,
                                                     PROPERTY_VALUE_HANDLE* phPropertyValue,
                                                     char* szPropertyName,
                                                     char* szPropertyDescription,
                                                     char* szPropertyValue,
                                                     char* szConfigValue );
   // Parameters:
   // hDevice - Handle to target device queue
   // phProperty - Pointer to handle for return of property handle that is requested to be changed.
   // phPropertyValue - Pointer to handle for return of property value handle that is requested to be changed.
   // szPropertyName - Return buffer for the name of the property that is requested to be changed.
   // szPropertyDescription - Return buffer for the description of the property that is requested to be changed.
   // szPropertyValue - String value of the property value that is requested to be changed.
   // szConfigValue - String configuration value of the property value that is requested to be changed.

   //
   // This function will return the property handle from the specified property value handle.
   PROPERTY_HANDLE (*pHB_GetPropertyHandleFromPropertyValueHandle)( PROPERTY_VALUE_HANDLE hPropertyValue );
   // Parameters:
   // hPropertyValue : Property Value handle whose property value is to be returned.

   //
   // This function will send the supplied interface argument pact to the devices configured
   // interface.
   HARDWARE_RC (*pHB_SendInterfacePackToInterface)( DEVICE_HANDLE hDevice, const InterfaceArgumentPack* pPack );
   // Parameters:
   // hDevice : Device handle of the device whose interface is to be sent the interface argument pack.
   // pPack : InterfaceArgument pack that contains interface specific parameters...

   //
   // This function will subscribe the Device to a "well known" Notification list.  Notification Lists enable Hardware
   // Interfaces to send notification data to interested Devices when changes are detected by the Hardware.
   void (*pHB_SubscribeToNotificationList)( DEVICE_HANDLE hDevice, LPCTSTR szListName, LPCTSTR szFilter );
   // Parameters :
   // hDevice : Device handle to subscribe
   // szListName : Well Known list name
   // szFilter : Additional information to help Devices filter their subscription.  For example, if an X10 device only
   //             required notifications when a particular House Code or Unit Code changed, it would subscribe with a
   //             filter value of "H=h;U=u" where h and u represent house code and unit code.  If there is no filter to
   //             specify, NULL can be passed.

   //
   // This function will remove the device from the specified subscription list (and optinally, filter).
   void (*pHB_UnSubscribeFromNotificationList)( DEVICE_HANDLE hDevice, LPCTSTR szListName, LPCTSTR szFilter );
   // Parameters:
   // hDevice : Device handle to subscribe
   // szListName : This is the same list name that would have been supplied earlier with pHB_SubscribeToNotificationList
   // szFilter : This is the same filter that would have been supplied earlier with pHB_SubscribeToNotificationList

   //
   // This function will return the device handle, enabled state, and name for the specified device description
   BOOL (*pHB_GetDeviceInfoFromDescription)( const char* szDeviceDescription,
                                             DEVICE_HANDLE* hDevice,
                                             BOOL* bEnabled,
                                             const char** szDeviceName );
   // Parameters:
   // szDeviceDescription : Device description whose information is being queried.
   // hDevice : Return pointer to device handle.
   // bEnabled : Return pointer to the device enabled state.
   // szDeviceName : Return pointer to the device name.

   //
   // This function will return the property handle and name for the specified property description
   BOOL (*pHB_GetPropertyInfoByDescription)( DEVICE_HANDLE hDevice,
                                             const char* szPropertyDescription,
                                             PROPERTY_HANDLE *phProperty,
                                             const char** szPropertyName );
   // Parameters:
   // hDevice : Device handle that owns the property.
   // szPropertyDescription : Description of the property to return information for.
   // phProperty : Return pointer to property handle.
   // szPropertyName : Return pointer to property name.

   //
   // This function will clear the dynamic property value array.
   BOOL (*pHB_ClearArrayPropertyValue)( PROPERTY_VALUE_HANDLE hPropertyValue );
   // Parameters :
   // hPropertyValue : Property value handle to clear.  This must be a 'dynamic' property value.

   //
   // This function will add the specified value to the dynamic property value array.
   BOOL (*pHB_AddArrayPropertyValue)( PROPERTY_VALUE_HANDLE hPropertyValue, const char* szValue );
   // Parameters :
   // hPropertyValue : Property value handle
   // szValue : New value to add to the end of the array.
   const char* (*pHB_GetArrayPropertyValueAsStringAtIndex)( PROPERTY_VALUE_HANDLE hPropertyValue, int nIndex );

   //
   // This function will return the state of the specified Mode.
   const char* (*pHB_GetModeStateByName)( const char* szModeName );
   // Parameters :
   // szModeName : Name of the mode whose state is requested.

   //
   // This function will change the state of the specified Mode.
   BOOL (*pHB_SetModeStateByName)( const char* szModeName, BOOL bActive );
   // Parameters :
   // szModeName : Name of the Mode to change state.
   // bActive : TRUE to set state as Active. FALSE to set state as Inactive.

   //
   // This function will execute the task specified.
   BOOL (*pHB_ExecuteTaskByName)( const char* szTaskName );
   // Parameters :
   // szTaskName : Name of the task to execute.

   //
   // This function will check the running state of a task.
   BOOL (*pHB_IsTaskRunning)( const char* szTaskName );
   // Parameters :
   // szTaskName - Name of the Task to check.

   //
   // This method will terminate the specified Task.
   void (*pHB_StopTaskByName)( const char* szTaskName );
   // Parameters:
   // szTaskName : Name of the task to  terminate.

   //
   // This function will return an array of WinMsg structures that are associated with the
   // specified property value. It returns TRUE if successful, or FALSE if not successful.
   BOOL (*pHB_GetWinMsgArrayAssociatedWithPropertyValue)( PROPERTY_HANDLE hProperty, PROPERTY_VALUE_HANDLE hPropertyValue, WinMsg* paMessages, DWORD* pdwBufferSize );
   // Parameters:
   // hProperty      : Property handle of owning property.
   // hPropertyValue : Property value needing the Windows Messages array.
   // paMessages     : Pointer to buffer to hold array of WinMsg structures upon return.  If this parameter is NULL, the function
   //                   will return the size in bytes needed in pdwBufferSize.
   // pdwBufferSize : Pointer to the size of the buffer passed in paMessages.  Upon return, this value
   //                   will contain the size of paMessages.

   //
   // This function will create a new property.  Returns TRUE if successful, or FALSE if not successful.
   BOOL (*pHB_CreateDeviceProperty)( const char* szPropertyName, const char* szPropertyDescription, ePropertyType ptType );
   // Parameters:
   // szPropertyName - Name of property.  Must be unique and not already exist.
   // szPropertyDescription - Default property description.  Can be changed by user, but must be unique

   //
   // This function will create a new property value.
   // If the value already existed, or if it was created, it will return TRUE.  If there was
   // an error creating the new property value, it will return FALSE.
   BOOL (*pHB_CreateDevicePropertyValue)( const char* szPropertyName, const char* szPropertyValue );
   // Parameters:
   // szPropertyName - Name of property.  Must match an existing Property Name in the system
   // szPropertyValue - Name of Property value to be associated with szPropertyName

   //
   // This function will map (assign) a property to a Device.  It is typically NOT
   // needed, since all registered properties are automatically mapped to a new device.
   // It is only needed when the specific device instance needs specific properties
   // that are not shared with all other device instances of the same type.
   // If there was an error creating the mapping, the function will return FALSE.
   // If there was an error creating the mapping, it may be due to the fact that the
   // Property does not exist.
   BOOL (*pHB_CreateDevicePropertyMapping)( DEVICE_HANDLE hDevice,
                                            const char* szPropertyName,
                                            const char* szPropertyDescription,
                                            const char* szInitialValue,
                                            eIOType ioType,
                                            BOOL bDefault,
                                            BOOL bPersistent,
                                            BOOL bAllowSameValueChange,
                                            BOOL bTraceToDevLog,
                                            BOOL bDynamic );
   // Parameters:
   // hDevice : Device Handle
   // szPropertyName - Name of property.  Must match pre-defined server property names.
   // szPropertyDescription - Default property description.  Can be changed by user.
   // szInitialValue - Initial value of the property.
   // ioType - Not currently used, but can/should be set correctly.
   // bDefault - TRUE if this is the default property for the device.  Otherwise it is FALSE.  Each device must have ONLY ONE default property.
   // bPersistent - TRUE if this value is to be saved to the DB on each change.  FALSE if the value is not to be saved.
   // bAllowSameValueChange - TRUE if this value is allowed to be changed to the same value.  FALSE if it can only be changed to a new value.
   // bTraceToDeviceLog - TRUE if this value should trace its property changes to the device log.  FALSE if it should not.
   // bDynamic - TRUE if this is a dynamic property.  NOTE:  Typically this is FALSE.

   //
   // This function will return the unique device instance ID.  It will return 0 on error.
   int (*pHB_GetDeviceID)( DEVICE_HANDLE hDevice );
   // Parameters:
   // hDevice : Device Handle

   //
   // This function will add a new value to the selectable list of valid values for the property.
   BOOL (*pHB_AddNewPropertyValue)( PROPERTY_HANDLE hProperty, const char* szNewValue );
   // Parameters:
   // hProperty - Handle to parent/owning property
   // szNewValue - New property value to add.

   //
   // This function will return an array of Command strings that are associated with the
   // specified property value.  It returns TRUE if successful, and FALSE if not successful.
   BOOL (*pHB_GetGenericCommandsAssociatedWithValue)( PROPERTY_HANDLE hProperty, PROPERTY_VALUE_HANDLE hPropertyValue, GenericCommand* paCommands, DWORD* pdwBufferSize );
   // Parameters:
   // hProperty      : Property handle of owning property.
   // hPropertyValue : Property value needing the Command array.
   // paCommands     : Pointer to buffer to hold array of GenericCommands upon return.  If this parameter is NULL, the function
   //                   will return the size in bytes needed in pdwBufferSize.
   // pdwBufferSize : Pointer to the size of the buffer passed in paCommands.  Upon return, this value
   //                   will contain the size of paCommands.

   //
   // This function will return send a query string to the configured hardware instance.
   // The query string must be a supported string that the hardware interface will recognize.
   BOOL (*pHB_QueryHardwareInstance)( DEVICE_HANDLE hDevice, const char* szQuery, void* pResponse );
   // Parameters:
   // hDevice     : Device Handle
   // szQuery     : Null terminated string that contains the query.
   // pResponse   : Pointer to query response buffer.  The specifics of the data type and format
   //                is well-known between the device and the interface.

   //
   // This function will indicate to the server that the Property Value change was neither
   // successful or failed.  This function is used to help manage temporary Property Values and
   // should not be used in response to Property Values changes.
   void (*pHB_ChangeRequestAbandoned)( PROPERTY_VALUE_HANDLE hPropertyValue );
   // Parameters:
   // hPropertyValue : Property value handle that was abandoned.

   //
   // This function will control the specitifed remote.  The control abilities match to a Task Action Properties.
   BOOL (*pHB_ControlRemote)( eClientControlAction ccaAction, const char* szWhichRemotes, const char* pArg1, const char* pArg2 );
   // Parameters:
   // ccaAction      : Specifies the type of client (remote) control.  See eClientControlAction enum at the top of this
   //                   header for possible options.
   // pArg1 & pArg2  : The meaning of pArg1 and pArg2 (optional argument data) depend on the client control action specified.  The optional
   //                   argument data is no required for all cases and shoule be NULL if not used.  See the Task Action Property window
   //                   for hints on what the optional argument data should be
   // szWhichRemotes : Set to NULL indicate "All Remotes".

   //
   // This function will return a string containing the list of Device Descriptions configured in the system.
   BOOL (*pHB_GetDeviceList)( char* pszDeviceList, DWORD* pdwBufferSize, const char* szDelimiter );
   // Parameters:
   // pszDeviceList : Pointer to buffer to hold the string of Device Descriptions.  If this parameter is NULL, the function
   //                   will return the size in bytes needed in pdwBufferSize.
   // pdwBufferSize : Pointer to the size of the buffer passed in pszDeviceList.  Upon return, this value
   //                   will contain the size of pszDeviceList.
   // szDelimiter   : Characters that will be used as delimiter to separate Device Descriptions in pszDeviceList.

   //
   // This function will return a string containing the list of Properties for the specified Device Description.
   BOOL (*pHB_GetPropertyListForDevice)( const char* pszDeviceDescription, char* pszPropertyList, DWORD* pdwBufferSize, const char* szDelimiter );
   // Parameters:
   // pszDeviceDescription : Description of Device to return the Property list for.
   // pszPropertyList : Pointer to buffer to hold the string of Property Descriptions.  If this parameter is NULL, the function
   //                   will return the size in bytes needed in pdwBufferSize.
   // pdwBufferSize : Pointer to the size of the buffer passed in pszPropertyList.  Upon return, this value
   //                   will contain the size of pszPropertyList.
   // szDelimiter   : Characters that will be used as delimiter to separate Property Descriptions in pszPropertyList.

   //
   // This function will return a string containing the list of Valid Property Values for the specified Property
   BOOL (*pHB_GetPropertyValueListForProperty)( const char* pszPropertyDescription, char* pszValueList, DWORD* pdwBufferSize, const char* szDelimiter );
   // Parameters:
   // pszPropertyDescription : Description of Property to return the Value list for.
   // pszValueList : Pointer to buffer to hold the string of Property Values.  If this parameter is NULL, the function
   //                   will return the size in bytes needed in pdwBufferSize.
   // pdwBufferSize : Pointer to the size of the buffer passed in pszValueList.  Upon return, this value
   //                   will contain the size of pszValueList.
   // szDelimiter   : Characters that will be used as delimiter to separate Property Values in pszValueList.

   //
   // This function will return a string containing the list of Tasks configured in the system.
   BOOL (*pHB_GetTaskList)( char* pszTaskList, DWORD* pdwBufferSize, const char* szDelimiter );
   // Parameters:
   // pszTaskList   : Pointer to buffer to hold the string of Tasks Names.  If this parameter is NULL, the function
   //                   will return the size in bytes needed in pdwBufferSize.
   // pdwBufferSize : Pointer to the size of the buffer passed in pszTaskList.  Upon return, this value
   //                   will contain the size of pszTaskList.
   // szDelimiter   : Characters that will be used as delimiter to separate Task Names in pszTaskList.

   //
   // This function will return a string containing the list of Modes configured in the system.
   BOOL (*pHB_GetModeList)( char* pszModeList, DWORD* pdwBufferSize, const char* szDelimiter );
   // Parameters:
   // pszModeList   : Pointer to buffer to hold the string of Modes Names.  If this parameter is NULL, the function
   //                   will return the size in bytes needed in pdwBufferSize.
   // pdwBufferSize : Pointer to the size of the buffer passed in pszModeList.  Upon return, this value
   //                   will contain the size of pszModeList.
   // szDelimiter   : Characters that will be used as delimiter to separate Mode Names in pszModeList.


   //
   // This function will register a device property.
   BOOL (*pHB_RegisterDevicePropertyUsingWizard)( DEVICE_MODULE_HANDLE hDeviceModule,
                                        int nDeviceNumber,
                                        const char* szPropertyName,
                                        const char* szPropertyDescription,
                                        BOOL bDefault,
                                        const char* szInitialValue,
                                        eIOType ioType,
                                        BOOL bPersistent,
                                        BOOL bAllowSameValueChange,
                                        BOOL bTraceToDeviceLog,
                                        BOOL bDynamic,
                                        const char* szWizardText );
   // Parameters:
   // hDeviceModule - Handle to device module to register property.
   // nDeviceNumber - Device number (ordinal) to register property for.  This matches nDeviceNumber ordinal returned in HBGetDeviceInfo().
   // szPropertyName - Name of property.  Must match pre-defined server property names.
   // szPropertyDescription - Default property description.  Can be changed by user.
   // bDefault - TRUE if this is the default property for the device.  Otherwise it is FALSE.  Each device must have ONLY ONE default property.
   // szInitialValue - Initial value of the property.
   // ioType - Not currently used, but can/should be set correctly.
   // bPersistent - TRUE if this value is to be saved to the DB on each change.  FALSE if the value is not to be saved.
   // bAllowSameValueChange - TRUE if this value is allowed to be changed to the same value.  FALSE if it can only be changed to a new value.
   // bTraceToDeviceLog - TRUE if this value should trace its property changes to the device log.  FALSE if it should not.
   // bDynamic - TRUE if this is a dynamic property.  NOTE:  Typically this is FALSE.
   // szWizardText - If not NULL, the Wizard will prompt for a value for this property when creating the new Device.  It will use the text
   //                specified in the wizard property prompt page.  If NULL, the Wizard will not prompt for a value for the property.

   //
   // This function will add a button to the Device configuration screen.
   BOOL (*pHB_RegisterDeviceButton)( DEVICE_MODULE_HANDLE hDeviceModule, int nDeviceNumber, int nButtonCallbackID, const char* szButtonHeading, const char* szButtonText );
   // Parameters:
   // hDeviceModule - Handle to device module to register property.
   // nDeviceNumber - Device number (ordinal) to register property for.  This matches nDeviceNumber ordinal returned in HBGetDeviceInfo().
   // nButtonCallbackID : Instance defined ID for the button.  Will be returned as a reference if the user presses the button.
   // szButtonHeading : Text that will be displayed over the button in the Deivce configuration screen.
   // szButtonText : Text that will be displayed on the button in the Device configuration screen.

   //
   // This function will return the next property value from the Device property change request queue.
   // It will also remove the change request from the queue.
   // Note:  This callback has been obsoleted by pHB_GetAndRemoveNextChangeRequestOnQueue2().
   BOOL (*pHB_GetAndRemoveNextChangeRequestOnQueue2)( DEVICE_HANDLE hDevice,
                                                      PROPERTY_HANDLE *phProperty,
                                                      PROPERTY_VALUE_HANDLE* phPropertyValue,
                                                      char* szPropertyName,
                                                      char* szPropertyDescription,
                                                      char* szPropertyValue,
                                                      int*  nPropertyValueLen,
                                                      char* szConfigValue,
                                                      int*  nConfigValueLen );
   // Parameters:
   // hDevice - Handle to target device queue
   // phProperty - Pointer to handle for return of property handle that is requested to be changed.
   // phPropertyValue - Pointer to handle for return of property value handle that is requested to be changed.
   // szPropertyName - Return buffer for the name of the property that is requested to be changed.
   // szPropertyDescription - Return buffer for the description of the property that is requested to be changed.
   // szPropertyValue - String value of the property value that is requested to be changed.
   // nPropertyValueLen - Length of the szPropertyValue Buffer.  Upon return, this will hold the actual lenght
   //                      of the buffer.  If the returned value is -1, the buffer was too small to hold
   //                      the entire Property Value.
   // szConfigValue - String configuration value of the property value that is requested to be changed.
   // nConfigValueLen - Length of the szConfigValue Buffer.  Upon return, this will hold the actual lenght
   //                      of the buffer.  If the returned value is -1, the buffer was too small to hold
   //                      the entire Property Config Value.
};

//
// Plugin information returned to server on HBDeviceModuleInit() call
struct ModuleInfo
{
   int          m_nModuleInfoSize;                     // Initialized at runtime with sizeof() the structure
   const char*  m_szModuleVersion;                     // Version number of the plugin
   const char*  m_szModuleName;                        // Name of the Plugin
};


//
// Device Module Export Definitions
extern "C"
{
   HOUSEBOT_DEVICE_API BOOL   HBDeviceModuleInit( const DeviceCallBackInfo* pCallBackInfo, const ModuleInfo** ppModuleInfo );
   HOUSEBOT_DEVICE_API BOOL   HBGetDeviceInfo( int nDeviceNumber, const DeviceInfo** ppDeviceInfo );
   HOUSEBOT_DEVICE_API BOOL   HBDeviceRegisterProperties( int nDeviceNumber );
   HOUSEBOT_DEVICE_API BOOL   HBDeviceInit( DEVICE_HANDLE hDevice, BOOL bEnabled, const char* szDeviceName, const char* szDeviceDescription );
   HOUSEBOT_DEVICE_API BOOL   HBDeviceRun( DEVICE_HANDLE hDevice, HANDLE hTerminate, HANDLE hPropertyChangeRequest );
   HOUSEBOT_DEVICE_API BOOL   HBDeviceSetEnabled( DEVICE_HANDLE hDevice, BOOL bEnabled );
   HOUSEBOT_DEVICE_API void   HBDisplayHelp( const char* szDeviceName );
   HOUSEBOT_DEVICE_API void   HBAboutBox( const char* szDeviceName );
   HOUSEBOT_DEVICE_API void   HBDeviceButtonPressNotification( DEVICE_HANDLE hDevice, int nButtonID );

   //
   // Only required for devices that need to be called from the hardware interface.
   HOUSEBOT_DEVICE_API BOOL   HBDeviceSubscriptionNotification( DEVICE_HANDLE hDevice, LPCTSTR szListName, LPCTSTR szFilter, const CDataPack* pDataPack );

   //
   // Only required for devices that contain dynamic (device managed) properties
   HOUSEBOT_DEVICE_API BOOL   HBDeviceInitDynamicProperty( DEVICE_HANDLE hDevice, PROPERTY_HANDLE hProperty );
   HOUSEBOT_DEVICE_API BOOL   HBDeviceSetDynamicProperty( DEVICE_HANDLE hDevice, PROPERTY_HANDLE hProperty, const char* szValue, int nIndex );
   HOUSEBOT_DEVICE_API BOOL   HBDeviceDeleteDynamicProperty( DEVICE_HANDLE hDevice, PROPERTY_HANDLE hProperty, int nIndex );
   HOUSEBOT_DEVICE_API BOOL   HBDeviceMoveItemDownDynamicProperty( DEVICE_HANDLE hDevice, PROPERTY_HANDLE hProperty, int nIndex );
   HOUSEBOT_DEVICE_API BOOL   HBDeviceMoveItemUpDynamicProperty( DEVICE_HANDLE hDevice, PROPERTY_HANDLE hProperty, int nIndex );
}
