// Copyright (c)2004 Paul Forgey
#include "StdAfx.h"
#include "DeviceModule.h"
#include "Device.h"

#include <cstdarg>
#include <cstdlib>
#include <cstring>
#include <algorithm>
#include <functional>

static void deleteInstance (DeviceModule::instances_type::value_type &v)
{
    delete v.second;
}

static void deleteString (char *s)
{
    delete[] s;
}

DeviceModule::DeviceModule (HANDLE dllInstance, const DeviceCallBackInfo *cbi, 
        const std::string &version, const std::string &name) :
    m_dllInstance (dllInstance),
    m_cbi (*cbi)
{
    m_moduleInfo.m_nModuleInfoSize = sizeof (ModuleInfo);
    m_moduleInfo.m_szModuleName = dupCString (name);
    m_moduleInfo.m_szModuleVersion = dupCString (version);
}

DeviceModule::~DeviceModule ()
{
    std::for_each (m_instances.begin (), m_instances.end (), deleteInstance);
    std::for_each (m_strings.begin (), m_strings.end (), deleteString);
}

const char *DeviceModule::dupCString (const std::string &rhs)
{
    char *cstr = new char[rhs.length () +1];
    std::memcpy (cstr, rhs.data (), rhs.length ());
    cstr[rhs.length ()] = 0;
    m_strings.push_back (cstr);
    
    return cstr;
}

int DeviceModule::addDevice (intptr_t context, 
        CreateDeviceProc createProc,
        const std::string &deviceName, 
        eDeviceType deviceType, 
        const std::string &interfaceSignatures, 
        eDeviceFlags deviceFlags, 
        const std::string &wizardDescription, 
        const std::string &wizardFinalText)
{
    m_devices.push_back (devices_type::value_type ());
    ModuleDeviceInfo &mdi = m_devices.back ();
    mdi.createProcData = context;
    mdi.createProc = createProc;
    mdi.name = deviceName;
    
    mdi.deviceInfo.m_nDeviceInfoSize = sizeof (DeviceInfo);
    mdi.deviceInfo.m_szDeviceName = dupCString (deviceName);
    mdi.deviceInfo.m_eDeviceType = deviceType;
    mdi.deviceInfo.m_szInterfaceSignatures = dupCString (interfaceSignatures);
    mdi.deviceInfo.m_dfFlags = deviceFlags;
    mdi.deviceInfo.m_szModuleWizardDescription = dupCString (wizardDescription);
    mdi.deviceInfo.m_szWizardFinalText = dupCString (wizardFinalText);
    
    return (int)m_devices.size () -1;
}

const DeviceInfo *DeviceModule::deviceInfo (int deviceNumber) const
{
    if (deviceNumber < 0 || deviceNumber >= (int)m_devices.size ())
        return NULL;
    return &(m_devices[deviceNumber].deviceInfo);
}

CDevice *DeviceModule::instanceFromHandle (DEVICE_HANDLE handle)
{
    instances_type::iterator i = m_instances.find (handle);
    if (i == m_instances.end ())
        return NULL;
    return i->second;
}

const CDevice *DeviceModule::instanceFromHandle (DEVICE_HANDLE handle) const
{
    instances_type::const_iterator i = m_instances.find (handle);
    if (i == m_instances.end ())
        return NULL;
    return i->second;
}

bool DeviceModule::initialize (DEVICE_HANDLE handle, bool isEnabled, 
        const std::string &name, const std::string &description)
{
    devices_type::const_iterator id;
    
    for (id = m_devices.begin (); id != m_devices.end (); ++id)
        if (id->name == name)
            break;
    if (id == m_devices.end ())
        return false;

    if (m_instances.find (handle) != m_instances.end ())
        return false; // XXX should log something

    CDevice *device = id->createProc (id->createProcData, handle, this, 
            isEnabled, name, description);
    if (!device || !device->DeviceInit ())
    {
        delete device;
        return false;
    }
    m_instances[handle] = device;
    return true;    
}

bool DeviceModule::setEnabled (DEVICE_HANDLE handle, bool isEnabled)
{
    CDevice *device = instanceFromHandle (handle);
    if (device != NULL)
    {
        device->SetEnabled (isEnabled);
        return true;
    }
    return false;
}

bool DeviceModule::buttonPress (DEVICE_HANDLE handle, int buttonId)
{
    CDevice *device = instanceFromHandle (handle);
    if (device != NULL)
    {
        device->ButtonPressNotification (buttonId);
        return true;
    }
    return false;
}

// things start to get ugly here so the rest of your code doesn't have to :) 
bool DeviceModule::registerProperty (int deviceNumber, const std::string &name,
        const std::string &description, bool isDefault, 
        const std::string &initialValue, eIOType ioType, bool isPersistent,
        bool allowSameValueChange, bool trace, bool isDynamic)
{
    if (m_cbi.pHB_RegisterDeviceProperty (m_cbi.m_hModuleHandle, deviceNumber,
            name.c_str (), description.c_str (), isDefault ? TRUE : FALSE,
            initialValue.c_str (), ioType, isPersistent ? TRUE : FALSE,
            allowSameValueChange ? TRUE : FALSE, trace ? TRUE : FALSE,
            isDynamic ? TRUE : FALSE))
        return true;
    return false;
}

bool DeviceModule::createProperty (const std::string &name, 
        const std::string &description, ePropertyType type)
{
    if (m_cbi.pHB_CreateDeviceProperty (name.c_str (), description.c_str (),
            type))
        return true;
    return false;
}
 
// there's an inline version taking a std::string value
bool DeviceModule::createPropertyValue (const std::string &name,
        const char *value)
{
    if (m_cbi.pHB_CreateDevicePropertyValue (name.c_str (), value))
        return true;
    return false;
}

bool DeviceModule::subscriptionNotification (DEVICE_HANDLE handle, 
        const std::string &list, const std::string &filter,
        const CDataPack *pack)
{
    CDevice *device = instanceFromHandle (handle);
    if (device && device->SubscriptionNotification (list, filter, pack))
        return true;
    return false;
}

bool DeviceModule::run (DEVICE_HANDLE handle, HANDLE terminateEvent,
        HANDLE propertyChangeEvent)
{
    CDevice *device = instanceFromHandle (handle);
    if (device && device->DeviceRun (terminateEvent, propertyChangeEvent))
        return true;
    return false;
}
