// ====================================================================
// Original file Copyright (c) 2002 - 2004 Meedio, LLC.  All rights reserved.
//
// File          : Property.h
// Origin Date   : 09/7/2002
// Purpose       : This file contains the implementations for the CProperty
//                   and CPropertyValue helper classes.
// ====================================================================
#include "stdafx.h"     // for MFC
#include "Property.h"
#include "Device.h"

#include <cstdlib>

// ====================================================================
// Method Name : CProperty
// Description : Constructor for CProperty object.  This constructor will
//                simply take the supplied information and initialize its
//                member variables.
// Parameters  : pDevice - Parent/Owning device object
//               szPropertyName - Name of property.
//               szPropertyDescription - Description of property.
//               hProperty - Property Handle.
// ====================================================================
CProperty::CProperty (const CDevice *device, const std::string &name,
        const std::string &description, PROPERTY_HANDLE handle) :
    m_device (device),
    m_handle (handle),
    m_name (name),
    m_description (description)
{
}

// ====================================================================
// Method Name : CProperty
// Description : Constructor for CProperty object.  This constructor will
//                call the server to lookup the property handle and description.
// Parameters  : pDevice - Parent/Owning device object
//               szPropertyName - Name of property.
// ====================================================================
CProperty::CProperty (const CDevice *device, const std::string &name) :
    m_device (device),
    m_handle (NULL),
    m_name (name)
{
    char description[MAX_PROPERTY_DESCRIPTION_LENGTH] = "";
    m_handle = device->GetCallBackInfo ()->
            pHB_GetDevicePropertyHandleFromName (device->GetDeviceHandle (),
                name.c_str (), description);
    m_description = description;
}

// ====================================================================
// Method Name : CProperty
// Description : Constructor for CProperty object.  This constructor will
//                call the server to get the property name and description.
// Parameters  : pDevice - Parent/Owning device object
//               hProperty - Property Handle.
// ====================================================================
CProperty::CProperty (const CDevice *device, PROPERTY_HANDLE handle) :
    m_device (device),
    m_handle (handle)
{
    char name[MAX_PROPERTY_NAME_LENGTH] = "";
    char description[MAX_PROPERTY_DESCRIPTION_LENGTH] = "";
    
    device->GetCallBackInfo ()->pHB_GetPropertyInfoFromHandle (handle, name,
        description);
    
    m_name = name;
    m_description = description;
}

// ====================================================================
// Method Name : ~CProperty
// Description : Destructor for CProperty object
// ====================================================================
CProperty::~CProperty()
{
}


// ====================================================================
// Method Name : GetPropertyValueAsString
// Description : This method will return the string value of the current
//                property value.
// Returns     : String value of the current property value of the property.
// ====================================================================
std::string CProperty::GetPropertyValueAsString (void) const
{
    CPropertyValue *value = GetPropertyValue ();
    // oof!  can't use rvi here
    std::string result = value->GetPropertyValue ();
    delete value;
    
    return result;
}

// ====================================================================
// Method Name : GetPropertyValue
// Description : This method will return the current property value object.
//                It will call to the server to get the latest value.
// Returns     : Pointer to CPropertyValue object, or NULL if the property
//                does not exist on the server.
// ====================================================================
CPropertyValue* CProperty::GetPropertyValue (void) const
{
    PROPERTY_VALUE_HANDLE valueHandle = NULL;
    const char *valueString = NULL;
    const char *configValue = NULL;
    CPropertyValue *value = NULL;

    if (m_device)
    {
        const DeviceCallBackInfo *cbi = m_device->GetCallBackInfo ();
        if (cbi)
        {
            if (cbi->pHB_GetDevicePropertyValueFromPropertyHandle (m_handle,
                    &valueHandle, &valueString, &configValue))
                value = new CPropertyValue (this, valueHandle, 
                    std::string (valueString), std::string (configValue));
                
        }
    }
    return value;
}

// ====================================================================
// Method Name : CreatePropertyValue
// Description : This method will create a new Property value object
//                with the initial value specified.
// Parameters  : szNewValue - New (or initial) value of the property.
// Returns     : Pointer to new CPropertyValue object if the server
//                was able to create a new value.  NULL if the server
//                could not create a new value.
// ====================================================================
CPropertyValue *CProperty::CreatePropertyValue (
        const std::string &newValue) const
{
    PROPERTY_VALUE_HANDLE valueHandle = m_device->GetCallBackInfo ()->
            pHB_CreatePropertyValue (m_handle, newValue.c_str ());
    if (valueHandle)
        return new CPropertyValue (this, valueHandle, newValue, std::string ());

    return NULL;
}


// ====================================================================
// Method Name : AddNewPropertyValue
// Description : This method will add a new Property value to the list of
//                valid values for the Property.  It is NOT like the
//                CreatePropertyValue() method, as it does not actually
//                create an instance of the PropertyValue.
// Parameters  : szNewValue - New selectable value for the property.
// Returns     : TRUE if successful.  FALSE if not successful.
// ====================================================================
bool CProperty::AddNewPropertyValue (const std::string &newValue)
{
    if (m_device->GetCallBackInfo ()->pHB_AddNewPropertyValue (m_handle, 
            newValue.c_str ()))
        return true;
    return false;
}

// ====================================================================
// Method Name : GetIRCodeArrayAssociatedWithValue
// Description : This method will return an array of IRCode positions (ID's)
//                that are associated with the specified property value.
// Parameters  : pValue - Property Value whose IRCode positions are needed
// Returns     : The return value is TRUE if successful, or FALSE if not successful.
//               If the return is TRUE, pRetArray will contain an Array of unsigned short
//               values that are the IRCode positions (ID's).
// ====================================================================
bool CProperty::GetIRCodeArrayAssociatedWithValue (const CPropertyValue *value,
        std::vector< unsigned short > *array) const
{
    array->clear ();
    DWORD sizeNeeded = 0;
    
    const DeviceCallBackInfo *cbi = m_device->GetCallBackInfo ();
    if (cbi)
    {
        if (cbi->pHB_GetIRCodeArrayAssociatedWithPropertyValue (m_handle,
                value->GetPropertyValueHandle (), NULL, &sizeNeeded))
        {
            array->resize (sizeNeeded / sizeof (unsigned short));
            if (cbi->pHB_GetIRCodeArrayAssociatedWithPropertyValue (m_handle,
                    value->GetPropertyValueHandle (), &(*array->begin ()), 
                    &sizeNeeded))
                return true;
        }
    }
    return false;
}


// ====================================================================
// Method Name : GetWinMsgArrayAssociatedWithValue
// Description : This method will return an array of WinMsg structures
//                that are associated with the specified property value.
// Parameters  : pValue - Property Value whose Windows Message definitions are needed.
//               paMessages - Pointer to buffer to hold array of WinMsg structures upon return.  If this parameter is NULL, the function
//                   will return the size in bytes needed in pdwBufferSize.
//               pdwBufferSize - Pointer to the size of the buffer passed in paMessages.  Upon return, this value
//                   will contain the size of paMessages.
// Returns     : TRUE if successful.  FALSE if not successful.
// ====================================================================
bool CProperty::GetWinMsgArrayAssociatedWithValue (const CPropertyValue *value,
        std::vector< WinMsg > *array) const
{
    array->clear ();
    DWORD sizeNeeded = 0;
    
    const DeviceCallBackInfo *cbi = m_device->GetCallBackInfo ();
    if (cbi)
    {
        if (cbi->pHB_GetWinMsgArrayAssociatedWithPropertyValue (m_handle,
                value->GetPropertyValueHandle (), NULL, &sizeNeeded))
        {
            array->resize (sizeNeeded / sizeof (WinMsg));
            if (cbi->pHB_GetWinMsgArrayAssociatedWithPropertyValue (m_handle,
                    value->GetPropertyValueHandle (), &(*array->begin ()),
                    &sizeNeeded))
                return true;
        }
    }
    
    return false;
}

// ====================================================================
// Method Name : GetGenericCommandsAssociatedWithValue
// Description : This method will return an array of Command strings
//                that are associated with the specified property value.
// Parameters  : pValue - Property Value whose Commands are needed.
//               paCommands - Pointer to buffer to hold array of GenericCommand structures upon return.  If this parameter is NULL, the function
//                   will return the size in bytes needed in pdwBufferSize.
//               pdwBufferSize - Pointer to the size of the buffer passed in paCommands.  Upon return, this value
//                   will contain the size of paCommands.
// Returns     : TRUE if successful.  FALSE if not successful.
// ====================================================================
bool CProperty::GetGenericCommandsAssociatedWithValue (const CPropertyValue *value,
        std::vector< GenericCommand > *array) const
{
    array->clear ();
    DWORD sizeNeeded = 0;
    
    const DeviceCallBackInfo *cbi = m_device->GetCallBackInfo ();
    if (cbi)
    {
        if (cbi->pHB_GetGenericCommandsAssociatedWithValue (m_handle,
                value->GetPropertyValueHandle (), NULL, &sizeNeeded))
        {
            array->resize (sizeNeeded / sizeof (GenericCommand));
            if (cbi->pHB_GetGenericCommandsAssociatedWithValue (m_handle,
                    value->GetPropertyValueHandle (), &(*array->begin ()),
                    &sizeNeeded))
                return true;
        }
    }
    
    return false;
}

//////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
// CPropertyValue
//////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////

// ====================================================================
// Method Name : CPropertyValue
// Description : Constructor for CPropertyValue
// Parameters  : pOwningProperty - Pointer to owning/parent property object
//               hPropValue - Property Value handle
//               szValue - String value of the property value
//               szConfigValue - String configuration value of the property value
// ====================================================================
CPropertyValue::CPropertyValue (const CProperty *property, 
        PROPERTY_VALUE_HANDLE valueHandle, const std::string &value, 
        const std::string &configValue) :
    m_valueHandle (valueHandle),
    m_property (property),
    m_value (value),
    m_configValue (configValue),
    m_inChangeRequest (false)
{
}

// ====================================================================
// Method Name : ~CPropertyValue
// Description : Destructor for CPropertyValue
// ====================================================================
CPropertyValue::~CPropertyValue()
{
    if (m_inChangeRequest)
    {
        ChangeRequestAbandoned ();
    }
}

// ====================================================================
// Method Name : GetNextPropertyValueOnChangeRequestQueue
// Description : This method will retreive and remove the next property
//                value that has changed from the change queue.
// Parameters  : pDevice - Device object whose queue has been updated.
// Returns     : Pointer to a CPropertyValue object that indicates the NEW
//                value that the user wants to change to.
//                Returns NULL if there are no change values on the queue.
// ====================================================================
CPropertyValue* CPropertyValue::GetNextPropertyValueOnChangeRequestQueue( 
        CDevice* pDevice )
{
   PROPERTY_HANDLE hProperty;
   PROPERTY_VALUE_HANDLE hPropertyValue;
   char  szPropertyName[ MAX_PROPERTY_NAME_LENGTH ] = {0};
   char  szPropertyDescription[ MAX_PROPERTY_DESCRIPTION_LENGTH ] = {0};

   int   nPropertyValueLen = -1;
   int   nPropertyConfigValueLen = -1;
   char  *szPropertyValue = NULL;
   char  *szPropertyConfigValue = NULL;

   for (int nRetry = 1; nRetry <= 100; nRetry++)
   {
      //
      // Check to see if the supplied buffers were big enough.  If either of the buffer 'len'
      // arguments are -1, it means that it wasn't big enough for the data.
      if (nPropertyValueLen == -1)
      {
         nPropertyValueLen = nRetry * 100;
         if (szPropertyValue)
         {
            free( szPropertyValue );
         }
         szPropertyValue = (char*)malloc( nPropertyValueLen );
      }
      if (nPropertyConfigValueLen == -1)
      {
         nPropertyConfigValueLen = nRetry * 100;
         if (szPropertyConfigValue)
         {
            free( szPropertyConfigValue );
         }
         szPropertyConfigValue = (char*)malloc( nPropertyConfigValueLen );
      }

      //
      // Call the server to get and remove the next value from the queue.
      if (pDevice->GetCallBackInfo()->pHB_GetAndRemoveNextChangeRequestOnQueue2( pDevice->GetDeviceHandle(),
                                                                                 &hProperty,
                                                                                 &hPropertyValue,
                                                                                 szPropertyName,
                                                                                 szPropertyDescription,
                                                                                 szPropertyValue,
                                                                                 &nPropertyValueLen,
                                                                                 szPropertyConfigValue,
                                                                                 &nPropertyConfigValueLen ))
      {
         //
         // Make sure the buffers were large enough.
         if ((nPropertyConfigValueLen == -1) ||
             (nPropertyValueLen == -1))
         {
            //
            // Try again.
            continue;
         }

         //
         // Find the property that this value belongs to.
         CProperty* pProperty = pDevice->GetPropertyFromHandle( hProperty );

         //
         // If we couldn't find the property in the device, add it to the device.
         if (!pProperty)
         {
            pProperty = pDevice->CreatePropertyByName( szPropertyName, szPropertyDescription, hProperty );
         }

         //
         // Next create and return the property value object.
         CPropertyValue* pNewValue = new CPropertyValue( pProperty, hPropertyValue, szPropertyValue, szPropertyConfigValue );

         free( szPropertyConfigValue );
         free( szPropertyValue );

         return( pNewValue );
      }
   }

   free( szPropertyConfigValue );
   free( szPropertyValue );

   //
   // Nothing has changed.
   return( NULL );
}

// ====================================================================
// Method Name : ClearArrayPropertyValue
// Description : This method will clear the dynamic property value array.
// Returns     : TRUE if successful.  FALSE if unsuccessful.
// ====================================================================
bool CPropertyValue::ClearArrayPropertyValue (void)
{
    if (m_property->GetOwningDevice ()->GetCallBackInfo ()->
            pHB_ClearArrayPropertyValue (m_valueHandle))
        return true;
    return false;
}   

// ====================================================================
// Method Name : AddArrayPropertyValue
// Description : This method will add the specified value to the dynamic
//                property value array.
// Parameters  : szValue - New value to add to the end of the array.
// Returns     : TRUE if successful.  FALSE if unsuccessful.
// ====================================================================
bool CPropertyValue::AddArrayPropertyValue( const std::string &value )
{
    if (m_property->GetOwningDevice ()->GetCallBackInfo ()->
            pHB_AddArrayPropertyValue (m_valueHandle, value.c_str ()))
        return true;
    return false;
}

// ====================================================================
// Method Name : GetArrayPropertyValueAsStringAtIndex
// Description : This method will return the property value for the
//                dynamic property at the specified array index.
// Parameters  : nIndex - Array index whose property value is to be returned.
// Returns     : Property value string at specified index, or NULL if index is invalid.
// ====================================================================
std::string CPropertyValue::GetArrayPropertyValueAsStringAtIndex (
    int index) const
{
    const char *valueString = m_property->GetOwningDevice ()->
            GetCallBackInfo ()->pHB_GetArrayPropertyValueAsStringAtIndex (
                m_valueHandle, index);
    if (valueString)
        return std::string (valueString);
    return std::string ();
}

// ====================================================================
// Method Name : ClonePropertyValue
// Description : This method creates a copy of this property value object.
// Returns     : New copy of property value object.
// ====================================================================
CPropertyValue* CPropertyValue::ClonePropertyValue (void) const
{
    return m_property->CreatePropertyValue (GetPropertyValue ());
}

// ====================================================================
// Method Name : ChangeRequest
// Description : This method issues a property change request.  Issuing
//                a property change request will add the supplied property
//                value to the device property change queue.
// ====================================================================
void CPropertyValue::ChangeRequest()
{
    if (m_inChangeRequest)
        ChangeRequestAbandoned ();
    m_property->GetOwningDevice ()->GetCallBackInfo ()->pHB_ChangeRequest (
            m_valueHandle);
    m_inChangeRequest = true;
}

// ====================================================================
// Method Name : ChangeRequestCompleted
// Description : This method will tell the server that the property has
//                changed succesfully and to change the value.
// ====================================================================
void CPropertyValue::ChangeRequestCompleted (void)
{
    m_property->GetOwningDevice ()->GetCallBackInfo ()->
            pHB_ChangeRequestCompleted (m_valueHandle);
    m_inChangeRequest = false;
}

// ====================================================================
// Method Name : ChangeRequestFailed
// Description : This method will tell the server that the property has
//                NOT changed succesfully and to NOT change the value.
// ====================================================================
void CPropertyValue::ChangeRequestFailed()
{
    m_property->GetOwningDevice ()->GetCallBackInfo ()->
            pHB_ChangeRequestFailed (m_valueHandle);
    m_inChangeRequest = false;
}

// ====================================================================
// Method Name : ChangeRequestAbandoned
// Description : This method will tell the server that the property has
//                NOT been either successful or failed.  The server
//                will release the Property Value memory.
// ====================================================================
void CPropertyValue::ChangeRequestAbandoned()
{
    m_property->GetOwningDevice ()->GetCallBackInfo ()->
            pHB_ChangeRequestAbandoned (m_valueHandle);
    m_inChangeRequest = false;
}

