// ====================================================================
// Original file Copyright (c) 2002-2004 Meedio, LLC.  All rights reserved.
//
// File          : Property.cpp
// Origin Date   : 09/7/2002
// Purpose       : This file contains the definintions for the CProperty
//                   and CPropertyValue helper classes.
// ====================================================================

#ifndef MEEDIO_PROPERTY_H
#define MEEDIO_PROPERTY_H

//
// Include common Device plugin definitions.
#include "DeviceAPI.h"

#include <vector>

//
// Forward declarations
class CDevice;
class CPropertyValue;

//////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
// CProperty
//////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
class CProperty
{
public:
    //
    // Various constructors.  Typically these are called only by the CDevice class.
    CProperty( const CDevice *device, const std::string &propertyName, 
        const std::string &propertyDescription, PROPERTY_HANDLE handle);
    CProperty( const CDevice *device, const std::string &propertyName );
    CProperty( const CDevice *device, PROPERTY_HANDLE handle);
    virtual ~CProperty();

    //
    // Return a CDevice pointer to the device that owns this property.
    inline const CDevice *GetOwningDevice(void) const
        { return( m_device ); }
        
    inline const std::string &GetPropertyName (void) const
        { return m_name; }
        
    inline const std::string &GetPropertyDescrption (void) const
        { return m_description; }
        
    inline PROPERTY_HANDLE GetPropertyHandle (void) const
        { return m_handle; }


    ////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////
    //           See implementation file (Property.cpp) for method descriptions
    ////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////
    std::string    GetPropertyValueAsString (void) const;
    CPropertyValue *GetPropertyValue (void) const;
    CPropertyValue *CreatePropertyValue (const std::string &) const;
    bool           AddNewPropertyValue (const std::string &);
    
    bool GetIRCodeArrayAssociatedWithValue (const CPropertyValue *value,
            std::vector< unsigned short > *) const;
    bool GetWinMsgArrayAssociatedWithValue (const CPropertyValue *value,
            std::vector< WinMsg > *) const;
    bool GetGenericCommandsAssociatedWithValue (const CPropertyValue *value,
            std::vector< GenericCommand > *) const;

private:
    // owning device
    const CDevice *m_device;
    // handle of this propery
    PROPERTY_HANDLE m_handle;
    // name of this property
    std::string m_name;
    // user supplied description of this property
    std::string m_description;
};


//////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
// CPropertyValue
//////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
class CPropertyValue
{
public:
    CPropertyValue (const CProperty *property, 
            PROPERTY_VALUE_HANDLE valueHandle, const std::string &value, 
            const std::string &configValue);
    virtual ~CPropertyValue();

    inline const std::string &GetPropertyValue (void) const
        { return m_value; }
        
    inline const std::string &GetPropertyValueConfigString (void) const
        { return m_configValue; }
        
    inline const CProperty *GetOwningProperty (void) const 
        { return m_property; }
        
    inline PROPERTY_VALUE_HANDLE GetPropertyValueHandle (void) const
        { return m_valueHandle; }

    ////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////
    //           See implementation file (Property.cpp) for method descriptions
    ////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////
    bool               ClearArrayPropertyValue (void);
    bool               AddArrayPropertyValue (const std::string &value);
    std::string        GetArrayPropertyValueAsStringAtIndex (int) const;
    CPropertyValue     *ClonePropertyValue (void) const;

    void               ChangeRequest ();

    //
    // !!!! Note !!!! After a property value has been retreived from the device
    // change request queue, the Device MUST call either ChangeRequestComplete(),
    // ChangeRequestFailed() or ChangeRequestAbandoned.  Not calling one of these
    // methods will result in memory leaks.
    void                 ChangeRequestCompleted();
    void                 ChangeRequestFailed();
    void                 ChangeRequestAbandoned();

    static CPropertyValue *GetNextPropertyValueOnChangeRequestQueue (
            CDevice *device);

private:
    // current value
    std::string m_value;
    // current configuration value
    std::string m_configValue;
    // property value handle
    PROPERTY_VALUE_HANDLE m_valueHandle;
    // owning CProperty
    const CProperty *m_property;
    // between ChangeRequest () and a satisfaction to it
    bool m_inChangeRequest;
};

#endif /* MEEDIO_PROPERTY_H */
