// Copyright (c) 2004 Paul Forgey
#include "stdafx.h"
#include "CommThread.h"
#include "KrellInstance.h"
#include "KrellStatus.h"

#include <algorithm>
#include <cstring>
#include <cerrno>
#include <process.h>

CommThread::CommThread (CKrellInstance *instance) :
    m_instance (instance),
    m_terminateEvent (NULL),
    m_thread (NULL)
{
}

CommThread::~CommThread ()
{
    if (m_thread != NULL)
        terminate ();
    if (m_terminateEvent != NULL)
        CloseHandle (m_terminateEvent);
}

bool CommThread::run (arg_type commHandle)
{
    if (m_thread != NULL)
    {
        terminate ();
    }
    else
    if (m_terminateEvent == NULL)
    {
        m_terminateEvent = CreateEvent (NULL, TRUE, FALSE, NULL);
        if (!m_terminateEvent)
        {
            m_instance->TraceMessage (ttError, 
                "Cannot create termination event: %s",
                m_instance->GetLastErrorAsString ().c_str ());
            return false;
        }
    }
    
    proc_arg_type *parg = new proc_arg_type (this, commHandle);
    m_thread = (HANDLE) _beginthreadex (NULL, 0, threadProc, parg, 0, NULL);
    if (m_thread == NULL)
    {
        m_instance->TraceMessage (ttError, "_beginthreadex failed: %s",
            std::strerror (errno));
        delete parg;
        return false;
    }
    
    return true;
}

void CommThread::terminate (void)
{
    if (!m_thread)
        return;
    SetEvent (m_terminateEvent);
    WaitForSingleObject (m_thread, INFINITE);
    CloseHandle (m_thread);
    m_thread = NULL;
}

unsigned __stdcall CommThread::threadProc (void *pv)
{
    proc_arg_type *parg = reinterpret_cast< proc_arg_type * > (pv);
    arg_type arg = parg->second;
    CommThread *me = parg->first;
    delete parg;
    
    return me->proc (arg);
}

unsigned CommThread::proc (arg_type commHandle)
{
    OVERLAPPED ov;
    std::memset (&ov, 0, sizeof (ov));

    if ((ov.hEvent = CreateEvent (NULL, TRUE, FALSE, NULL)) == NULL)
    {
        m_instance->TraceMessage (ttError, 
            "unable to create overlapped wait event: %s",
            m_instance->GetLastErrorAsString ().c_str ());
        return 0;
    }

    HANDLE waitvec[2] = { m_terminateEvent, ov.hEvent };
    DWORD error, xfer, ptr;
    BYTE buffer[sizeof (KrellStatus)];
    ptr = 0;
    
    m_instance->TraceMessage (ttDebug, "status thread started");

    for (;;)    
    {
        if (!ReadFile (commHandle, buffer + ptr, sizeof (buffer) - ptr, NULL, 
                &ov))
        {
            error = GetLastError ();
            if (error != ERROR_IO_PENDING)
            {
                m_instance->TraceMessage (ttError, "ReadFile: %08x", error);
                CloseHandle (ov.hEvent);
                return 0;
            }
        }

        if (WaitForMultipleObjects (2, waitvec, FALSE, INFINITE) == 
                WAIT_OBJECT_0)
        {
            CancelIo (commHandle);
            GetOverlappedResult (commHandle, &ov, &xfer, FALSE);
            
            CloseHandle (ov.hEvent);

            m_instance->TraceMessage (ttDebug, 
                    "status thread terminating normally");
            return 0;
        }
        
        if (!GetOverlappedResult (commHandle, &ov, &xfer, FALSE))
        {
            m_instance->TraceMessage (ttError, "GetOverlappedResult: %s", 
                    m_instance->GetLastErrorAsString ().c_str ());
            CloseHandle (ov.hEvent);
            return 0;
        }
        ResetEvent (ov.hEvent);

        m_instance->TraceMessage (ttDebug, "received %d bytes from port", xfer);
        
        ptr += xfer;
        if (ptr < sizeof (buffer))
        {
            // partial read
            const BYTE *mark = buffer+ptr;
            while (--mark > buffer && *mark != 0x55)
                ;
            if (mark <= buffer)
                // still in the middle of nowhere
                continue;
            ptr -= (mark-buffer);
            std::memmove (buffer, mark, ptr);
            continue;
        }
        
        m_instance->TraceMessage (ttDebug, "updated status");
        
        ptr = 0;
        m_instance->UpdateStatus ((const KrellStatus *) buffer);
        
    }
    
    return 0; // unreached

}
