// ====================================================================
// Copyright (c) 2004 Paul Forgey
// original file Copyright (c) 2002 - 2004 Meedio, LLC.  All rights reserved.
//    This file is provided as part as a software development kit (SDK).
//    This file can be used, royalty free, in any form to extend the
//    functionality of the HouseBot product.
//
// File          : KrlelInstance.cpp
// Origin Date   : 11/05/2004
// derived from TemplateInstance.cpp 09/7/2002
// Purpose       : This file contains the implementation of the
//                   CKrellInstance for talking to a Krell Showcase or 
//                   HTS 71 home theater controller.
// ====================================================================

#include "stdafx.h"
#include "KrellInstance.h"
#include "../../syCommon/InterfaceAPI.h"
#include "../../syCommon/DataPack.h"

#include <cstring>
#include <algorithm>

#pragma warning (disable:4355) // we know what we are doing

// ====================================================================
// Method Name : CKrellInstance
// Description : CKrellInstance constructor
// Parameters  : hInstance - Hardware Instance Handle passed by HBRegisterProperties()
//                pCallBackInfo - Copy of callback info structure passed by HBModuleInit()
// ====================================================================
CKrellInstance::CKrellInstance( HARDWARE_INSTANCE_HANDLE hInstance, const CallBackInfo* cbi) :
    CHardwareInstance( hInstance, cbi ),
    m_bEnabled( FALSE ),
    m_serialDevice (INVALID_HANDLE_VALUE),
    m_statusReader (this)
{
    std::memset (&m_status, 0, sizeof (m_status));
    std::memset (&m_writeOV, 0, sizeof (m_writeOV));
}

CKrellInstance::~CKrellInstance ()
{
    CloseComPort ();

    if (m_writeOV.hEvent != NULL)
        CloseHandle (m_writeOV.hEvent);
}

void CKrellInstance::Terminate (void)
{
    CloseComPort ();
}

void CKrellInstance::UpdateStatus (const KrellStatus *status)
{
    m_status = *status;
    CDataPack pack;
    pack.AddData ("Status", status, sizeof (*status));
    NotifySubscribedDevices (kKrellNotification_Status, "", &pack);
}

// see comments in header file explaining why we block
bool CKrellInstance::SendData (const char *string)
{
    TraceMessage (ttDebug, ">>> '%s'", string);

    if (m_serialDevice == INVALID_HANDLE_VALUE)
    {
        TraceMessage (ttDebug, "requested to send data, but port is not open");
        return false;
    }

    if (m_writeOV.hEvent == NULL)
    {
        if ((m_writeOV.hEvent = CreateEvent (NULL, TRUE, FALSE, NULL)) == NULL)
        {
            TraceMessage (ttError, "Cannot create overlapped event (writing): %08x",
                    GetLastError ());
            return false;
        }
    }
    
    DWORD error;
    ResetEvent (m_writeOV.hEvent);
    if (!WriteFile (m_serialDevice, string, strlen (string), NULL, &m_writeOV))
    {
        error = GetLastError ();
        if (error == ERROR_IO_PENDING)
        {
            // we actally don't care about numberOfBytesWritten
            if (!GetOverlappedResult (m_serialDevice, &m_writeOV, &error, TRUE))
            {
                TraceMessage (ttError, "Error catching overlapped write: %08X",
                        GetLastError ());
                return false;
            }
        }
        else
        {
            TraceMessage (ttError, "Error writing to device: %08X",
                    GetLastError ());
            return false;
        }
    }
    
    return true;
}

void CKrellInstance::CloseComPort (void)
{
    m_statusReader.terminate ();
    if (m_serialDevice != INVALID_HANDLE_VALUE)
    {
        CloseHandle (m_serialDevice);
        m_serialDevice = INVALID_HANDLE_VALUE;
    }
}

bool CKrellInstance::OpenComPort (void)
{
    CloseComPort ();

    HANDLE com = CreateFile (m_serialName.c_str (), GENERIC_READ | GENERIC_WRITE, 0, NULL, 
        OPEN_EXISTING, FILE_FLAG_OVERLAPPED, NULL);
    if (com == INVALID_HANDLE_VALUE)
    {
        TraceMessage (ttError, "Cannot open serial port '%s': %08x",
                m_serialName.c_str (), GetLastError ());
        return false;
    }
    
    // put a one second timeout on a write operation just in case something
    // very strange happens
    COMMTIMEOUTS tmo = { 100, 0, 0, 0, 1000 };
    if (!SetCommTimeouts (com, &tmo))
    {
        TraceMessage (ttError, "Error setting COM timeouts (is this a serial port?): %08x",
                GetLastError ());
        CloseHandle (com);
        return false;
    }
    
    DCB dcb;
    if (!GetCommState (com, &dcb))
    {
        TraceMessage (ttError, "Error getting COM state: %08x", GetLastError ());
        CloseHandle (com);
        return false;
    }

    // 8-1-N, no handshaking
    dcb.BaudRate = CBR_9600;
    dcb.ByteSize = 8;
    dcb.Parity = NOPARITY;
    dcb.StopBits = ONESTOPBIT;
    dcb.fDsrSensitivity = FALSE;
    dcb.fOutxCtsFlow = FALSE;
    dcb.fOutxDsrFlow = FALSE;
    dcb.fDtrControl = DTR_CONTROL_ENABLE;
    dcb.fRtsControl = RTS_CONTROL_ENABLE;
    dcb.fAbortOnError = FALSE;
    
    if (!SetCommState (com, &dcb))
    {
        TraceMessage (ttError, "Error setting COM state: %08x", GetLastError ());
        CloseHandle (com);
        return false;
    }

    if (!m_statusReader.run (com))
    {
        CloseHandle (com);
        return false;
    }
    
    std::swap (com, m_serialDevice);

    if (!SendData ("ASTEZSTAZ"))
    {
        CloseComPort ();
        return false;
    }
    
    return true;
}

// ====================================================================
// Method Name : RegisterProperties
// Description : This method is called by the server to allow the
//                instance to register all of its properties.
// Returns     : TRUE if successful.  FALSE if unsuccessful.
// ====================================================================
bool CKrellInstance::RegisterProperties (void)
{
    //
    // Register the properties.
    const char *aValues[] = { "Yes", "No", NULL };
    RegisterHardwareModuleProperty( "Enabled", ptToggle, aValues, "No", true, "" );
    RegisterHardwareModuleProperty ("Com Port", ptString, NULL, "COM1:", true, 
            "Select the COM Port that the Hardware is attached to.");

    //
    // Get the current state of the property values.
    std::string value = GetPropertyValueFromName ("Com Port");
    if (!value.empty ())
        SetComPort (value);

    value = GetPropertyValueFromName ("Enabled");
    if (value == "Yes")
    {
        if (!Enable (true))
            SetPropertyValueFromName ("Enabled", "No");
    }
    else
    {
        Enable (false);
    }

    return true;
}


// ====================================================================
// Method Name : UpdatePropertyValue
// Description : This method is called by the server when the user
//                changes a property value.
// Parameters  : szPropertyName - Name of property that has changed.
//               szNewValue - New value of the property.
// Returns     : Returns TRUE if changed successfully.  Returns FALSE if not changed successfully.
// ====================================================================
bool CKrellInstance::UpdatePropertyValue (const std::string &propertyName, 
        const std::string &propertyValue )
{
    TraceMessage (ttDebug, "setting '%s' to '%s'", 
            propertyName.c_str (), propertyValue.c_str ());

    //
    // See if it is the "Enabled" property.
    if (propertyName == "Enabled" )
    {
        if (propertyValue == "Yes")
        {
            if (!Enable (true))
                return false;
        }
        else
        {
            Enable (false);
        }
        return true;
    }
    else if (propertyName == "Com Port")
    {
        if (!SetComPort (propertyValue))
            SetPropertyValueFromName ("Enabled", "No");
        return true;
    }

    return false;
}

bool CKrellInstance::Enable (bool enabled)
{
    if (!enabled || m_serialName.empty ())
    {
        CloseComPort ();
        m_bEnabled = false;
    }
    else
    if (!m_serialName.empty ())
    {
        m_bEnabled = OpenComPort ();
    }
    return m_bEnabled;
}

bool CKrellInstance::SetComPort (const std::string &port)
{
    m_serialName = port;
    if (m_bEnabled)
        return OpenComPort ();
    return false;
}


// ====================================================================
// Method Name : PropertyButtonClick
// Description : This method is called by the server when the user
//                presses a property button
// Parameters  : nButtonCallbackID - Button ID passed in RegisterHardwareModuleButton()
//                that identifies the button pressed.
// ====================================================================
bool CKrellInstance::PropertyButtonClick( int nButtonCallbackID )
{
    return false;
}


// ====================================================================
// Method Name : Query
// Description : This function allows a Device to query the interface for
//                information.  The query and response are pre-established
//                format and types.
// Parameters  : szQuery - Null terminated string that specifies the query.
//               pResponse - Void pointer to a pre-established data type that
//                will provide the response data from the query.
// Notes       : This function is not meant as an ad-hoc method for
//                interrogating the interface.  It is only used for Devices
//                to communicate with Interfaces that they know well.
// ====================================================================
bool CKrellInstance::Query (const std::string &query, void *response)
{
    if (query == kKrellQuery_GetStatus)
    {
        std::memcpy (response, &m_status, sizeof (KrellStatus));
        return true;
    }

    return CHardwareInstance::Query (query, response);
}


////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
// Template specific exports
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////

HARDWARE_RC CKrellInstance::Command (const InterfaceArgumentPack *pPack)
{
    if (!m_bEnabled)
    {
        TraceMessage (ttError, "Attempt to control disabled device");
        return hrcFailureAbort;
    }

    if (m_serialDevice == INVALID_HANDLE_VALUE)
    {
        TraceMessage (ttError, "Do not have a valid serial device open");
        return hrcFailureAbort;
    }
    
    if (!pPack->correctVersion ())
    {
        TraceMessage (ttError, "Interface pack is wrong version");
        return hrcFailureAbort;
    }

    if (pPack->size () != 1 || (*pPack)[0].type () != InterfaceArgument::kType_String)
    {
        TraceMessage( ttError, 
            "Invalid number of arguments or type of argument supplied to KrellControl() interface");
        return( hrcFailureAbort );
    }

    if (!SendData ((*pPack)[0].stringValue ()))
    {
        TraceMessage (ttError, "send code failed");
        CloseComPort ();
        SetPropertyValueFromName ("Enabled", "No");
        return hrcFailureAbort;
    }
    
    return hrcSuccess;
}
