// ====================================================================
// Copyright (c) 2004 Paul Forgey
// original file Copyright (c) 2002 -2004 Meedio, LLC.  All rights reserved.
//    This file is provided as part as a software development kit (SDK).
//    This file can be used, royalty free, in any form to extend the
//    functionality of the HouseBot product.
//
// File          : KrellInterface.cpp
// Origin Date   : 11/05/2004
// derived from HardwareTemplate.cpp 09/04/2002
// Purpose       : Main module for sample Hardware Interface plugin.
// ====================================================================

#include "StdAfx.h"
#include "KrellInstance.h"
#include "../../Devices/KrellDevice/KrellProperties.h"
#include "../syCommon/HardwareAPI.h"
#include "../../syCommon/InterfaceAPI.h"

#include <cstdlib>
#include <map>

#include "resource.h"

// Collection of all instances of CKrellInstance
typedef  std::map< HARDWARE_INSTANCE_HANDLE, CKrellInstance* >     HARDWARE_INSTANCE_MAP;
static HARDWARE_INSTANCE_MAP  m_mapInstances;

// ====================================================================
// Method Name : DllMain
// Description : Standard Windows DLL entry point.
// ====================================================================
BOOL APIENTRY DllMain( HANDLE hModule, DWORD  ul_reason_for_call, LPVOID lpReserved )
{
   switch (ul_reason_for_call)
   {
      case DLL_PROCESS_ATTACH:
      case DLL_THREAD_ATTACH:
      case DLL_THREAD_DETACH:
      case DLL_PROCESS_DETACH:
         break;
   }
   return (TRUE);
}

//
// Plugin Module information structure.
static ModuleInfo  g_ModuleInfo =
{  sizeof (ModuleInfo),
// Version of the plugin.
   "1.0", 
// Name of the plugin.
   "Krell Hardware Module",
// Text that will appear in the wizard when user select the interface
   "RS-232 control of Showcase and HTS 7.1",
// Text that shows at the end of the wizard.
   "Krell Hardware Interface Setup Complete!",
// Specifies that there are Properties that the Wizard should setup when configuring the Interface.
   TRUE                             
};

//
// Globally stored callback info structure that is used by all instances to call
// server services.
CallBackInfo               g_CallBackInfo;

// ====================================================================
// Method Name : HBModuleInit
// Description : This function is called by the server only once on initial load
//                of the plugin.  It requests information about the plugin and
//                provides server callback information.
// Parameters  : CallBackInfo* - A pointer to a CallBackInfo structure.  This
//                structure includes the server version number and pointers to
//                all server callback functions.  This structure should be stored
//                and used for all instances of the hardware interface.
//                == Do not store the pointer to the structure.  Make a copy of the data. ==
//               ModuleInfo** - A pointer to a pointer to allow the plugin to return
//                to the server a ModuleInfo structure that gives general information about
//                the plugin.  The m_nModuleInfoSize member must be initialized to the sizeof()
//                the structure.  This allows for backward compatibility if the structure
//                is amended in the future.
// Returns     : Returns TRUE if successful.  Returns FALSE if unsuccessful.
// ====================================================================
HOUSEBOT_HARDWARE_API BOOL HBModuleInit( const CallBackInfo* pCallBackInfo, 
                                         const ModuleInfo** ppModuleInfo )
{
   memcpy( &g_CallBackInfo, pCallBackInfo, sizeof( CallBackInfo ) );
   *ppModuleInfo = &g_ModuleInfo;

   return( TRUE );
}

// ====================================================================
// Method Name : HBHardwareInstanceInit
// Description : The server calls this function to allow the plugin to initialize
//                the specified instance.
// Parameters  : hInstanceHandle - Instance handle to initialize.
// Returns     : Returns TRUE if the instance has initialized property.  Returns
//                FALSE if there was a problem with initialization.
// Notes       : This function is called when the server wants the instance to initialize
//                itself.  For example, if the instance connects to hardware via a RS232
//                port, this would be when to open the COM port and initialize the hardware.
// ====================================================================
HOUSEBOT_HARDWARE_API BOOL HBHardwareInstanceInit( HARDWARE_INSTANCE_HANDLE hInstanceHandle )
{
   //
   // Simply pass the call to the instance class (bool -> BOOL)
   return( m_mapInstances[ hInstanceHandle ]->Init() ? TRUE : FALSE );
}

// ====================================================================
// Method Name : HBHardwareInstanceTerminate
// Description : The server calls this function to allow the plugin to terminate
//                the specified instance.
// Parameters  : hInstanceHandle - Instance handle to terminate.
// Returns     : Returns TRUE if terminated successfully.  Returns FALSE if not terminated successfully.
// Notes       : This function is called when the server wants the instance to terminate.  For example,
//                if the instance connects to hardware via a RS232 port, this would be when to
//                close the COM port.
// ====================================================================
HOUSEBOT_HARDWARE_API BOOL HBHardwareInstanceTerminate( HARDWARE_INSTANCE_HANDLE hInstanceHandle )
{
    //
    // Get the instance object from the map of instances.
    HARDWARE_INSTANCE_MAP::iterator i = m_mapInstances.find (hInstanceHandle);
    if (i == m_mapInstances.end ())
        return FALSE;

    //
    // Pass the terminate call to the instance.
    i->second->Terminate();

    // Delete the instance object.
    delete (i->second);
    // Remove the instance object from the map.
    m_mapInstances.erase (i);

    return TRUE;
}


// ====================================================================
// Method Name : HBRegisterProperties
// Description : The server calls this function to allow the plugin to register
//                properties for a particular instance.  All properties should
//                be registered when this function is called.  Do not register
//                properties before this function is called.
// Parameters  : hInstanceHandle - Instance handle
// Returns     : Returns TRUE if the instance has successfully registered its
//                properties.  Returns FALSE if there was an error initializing
//                its properties.
// Notes       : This is the first call from the server that specifies an Instance
//                Handle.  It is important for the plugin to store this handle for
//                future reference and distinguish between handles on subsequent calls.
//                This function will be called once for each configured Hardware
//                Interface.  When called, the plugin should register its properties
//                using the pHB_RegisterHardwareModuleProperty or
//                pHB_RegisterHardwareModuleButton callback functions.  After properties
//                are registered, they can be viewed and modified from the server.
//                Properties can also be registered to NOT be shown in the user interface,
//                allowing the plugin to store private data in the server database.
// ====================================================================
HOUSEBOT_HARDWARE_API BOOL HBRegisterProperties( HARDWARE_INSTANCE_HANDLE hInstanceHandle )
{
    //
    // This is the first function that is called with a unique Instance Handle.
    // We must create an instance of the hardware interface and store it for future
    // per-Instance references.
    //

    //
    // Create a new Instance of our Template Hardware Interface.
    CKrellInstance* pInstance = new CKrellInstance( hInstanceHandle, &g_CallBackInfo );

    //
    // Add the instance to the map.  The map is keyed by the instance handle.
    m_mapInstances[ hInstanceHandle ] = pInstance;
    //
    // Call the RegisterProperties() of the instance object.
    return( m_mapInstances[ hInstanceHandle ]->RegisterProperties() );
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////
// The InterfaceInfo structures below setup the interface definitions for
// the interfaces supported by the plugin.
//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////
InterfaceInfo  g_InterfaceKrellCommand = {
   sizeof (InterfaceInfo),                  // Size of the structure.  Will be initialized at runtime.
   kKrellSignature_Command,                 // Signagure of the Interface.
   "KrellCommand",                          // Name of exported function to support the interface.
   "Sends commands to the unit.",           // Help text.
   hfNoFlags                                // Additional flags
};

// ====================================================================
// Method Name : HBHardwareGetInterfaceInfo
// Description : The server calls this function to query the plugin for information
//                on each of its supported interfaces.  This function is called
//                multiple times to individually query for each interface.
// Parameters  : int nInterfaceNumber - Specifies the interface number (ordinal)
//                that the server is requesting information about.  This always
//                starts with 0 and continues to be called with incrementing
//                nInterfaceNumbers until FALSE is returned due to no more
//                interfaces to report.
//               InterfaceInfo** - A pointer to a pointer to allow the plugin to
//                return to the server an InterfaceInfo structure that gives specific
//                information about the interface referenced by nInterfaceNumber.
//                This information is displayed at the server by clicking on the name
//                of the plugin in the "Hardware Interfaces" property view.  The
//                m_nInterfaceInfoSize member must be initialized to the sizeof() the
//                structure.  This allows for backward compatibility if the structure
//                is amended in the future
// Returns     : Returns TRUE if interface number exists.  Returns FALSE if interface
//                number does not exist.
// Notes       : The nInterfaceNumber has no real significance.  You can assign any
//                interface to any number as long as they start at zero and count up
//                sequentially.  If the plugin does not export ANY interfaces, simply
//                return FALSE from this function.
// ====================================================================
HOUSEBOT_HARDWARE_API BOOL HBHardwareGetInterfaceInfo( int nInterfaceNumber, const InterfaceInfo** ppInterfaceInfo )
{
    if (nInterfaceNumber == 0)
    {
        *ppInterfaceInfo = &g_InterfaceKrellCommand;
        return TRUE;
    }
    
    return( FALSE );
}

// ====================================================================
// Method Name : HBUpdatePropertyValue
// Description : The server calls this function when a property value has been changed.
// Parameters  : hInstanceHandle - Instance handle whose property value has changed.
//               szPropertyName - Name of property that has changed.
//               szNewValue - New value of the property.
// Returns     : Returns TRUE if changed successfully.  Returns FALSE if not changed successfully.
// Notes       : This function is called when the user changes the value of a property from the
//                servers user interface.  The plugin should recognize the various property names
//                and take appropriate actions.  For example, if the "COM Port" property value has
//                changed to a new port, the instance should close the old COM port and open the
//                new one.
// ====================================================================
HOUSEBOT_HARDWARE_API BOOL HBUpdatePropertyValue( HARDWARE_INSTANCE_HANDLE hInstanceHandle, const char* szPropertyName, const char* szNewValue )
{
   //
   // Simply pass the call along to the instance object.
   return( m_mapInstances[ hInstanceHandle ]->UpdatePropertyValue( szPropertyName, szNewValue ) );
}

// ====================================================================
// Method Name : HBPropertyButtonClick
// Description : The server calls this function when the user has pressed a property button.
// Parameters  : hInstanceHandle - Instance handle whose button was pressed.
//               nButtonCallbackID - ID of the button that was pressed.  This corresponds to
//                the button ID passed in pHB_RegisterHardwareModuleButton()
// Notes       : This function is called when the user presses a button in the configuration
//                screen for the plugin instance.
// ====================================================================
HOUSEBOT_HARDWARE_API BOOL HBPropertyButtonClick( HARDWARE_INSTANCE_HANDLE hInstanceHandle, int nButtonCallbackID )
{
   //
   // Simply pass the call along to the instance object.
   return( m_mapInstances[ hInstanceHandle ]->PropertyButtonClick( nButtonCallbackID ) );
}

// ====================================================================
// Method Name : HBDisplayHelp
// Description : The server calls this function when the user requests help.
// Notes       : The HBDisplayHelp() function should display help for
//                the plugin.  The help can be in any form, but it is
//                suggested, for consistency, that the help be in
//                HTMLHelp format.
// ====================================================================
#if 0
HOUSEBOT_HARDWARE_API void HBDisplayHelp()
{
   //
   // Display the Plugin help file (instead of the message box).
   MessageBox( NULL, "Plugin On-line Help", "Help", MB_OK );
}
#endif


// ====================================================================
// Method Name : HBQueryInstance
// Description : This function allows a Device to query the interface for
//                information.  The query and response are pre-established
//                format and types.
// Parameters  : hInstanceHandle - Instance handle whose button was pressed.
//               szQuery - Null terminated string that specifies the query.
//               ppResponse - Void pointer to a pre-established data type that
//                will provide the response data from the query.
// Notes       : This function is not meant as an ad-hoc method for
//                interrogating the interface.  It is only used for Devices
//                to communicate with Interfaces that they know well.
// ====================================================================
HOUSEBOT_HARDWARE_API BOOL HBQueryInstance( HARDWARE_INSTANCE_HANDLE hInstanceHandle, const char* szQuery, void* pResponse )
{
   return( m_mapInstances[ hInstanceHandle ]->Query( szQuery, pResponse ) );
}


///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////
// Template specific exports
///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////
extern "C" HOUSEBOT_HARDWARE_API HARDWARE_RC KrellCommand( HARDWARE_INSTANCE_HANDLE hInstanceHandle, InterfaceArgumentPack* pPack )
{
   //
   // Simply pass the call along to the instance object.
   return( m_mapInstances[ hInstanceHandle ]->Command ( pPack ) );
}

