// ====================================================================
// Original file Copyright (c) 2002 - 2004 Meedio, LLC.  All rights reserved.
//    This file is provided as part of a software development kit (SDK).
//    This file can be used, royalty free, in any form to extend the
//    functionality of the HouseBot product.
//
// File:           HardwareInstance.cpp
// Origin Date:    9/4/2002
// Platform:       Developed using MS VC++ 6.0
// Purpose:
//    This file contains the class implementation for CHardwareInterface.
//    The CHardwareInterface class is a helper class for writing HouseBot
//    Hardware plugin Interfaces.  The class wraps the server callback
//    function pointers for easier calling.
// ====================================================================

#include "stdafx.h"
#include "HardwareInstance.h"
#include "../../syCommon/InterfaceAPI.h"

#include <string>
#include <cstdlib>
#include <algorithm>

// ====================================================================
// Method Name : CHardwareInstance
// Parameters  : hInstance - Hardware Instance Handle passed by HBRegisterProperties()
//                pCallBackInfo - Copy of callback info structure passed by HBModuleInit()
// Description : Constructor for the HardwareInterface helper class.  Derive hardware instances
//                from this class to help manage server callbacks.
// ====================================================================
CHardwareInstance::CHardwareInstance (HARDWARE_INSTANCE_HANDLE instance, 
        const CallBackInfo *cbi) :
   m_instance (instance),
   m_cbi (*cbi)
{
}

CHardwareInstance::~CHardwareInstance ()
{
}

// virtual method defaults
bool CHardwareInstance::Init (void)
{
    return true;
}

void CHardwareInstance::Terminate (void)
{
}

bool CHardwareInstance::RegisterProperties (void)
{
    return true;
}

bool CHardwareInstance::UpdatePropertyValue (const std::string &,
        const std::string &)
{
    return false;
}

bool CHardwareInstance::PropertyButtonClick (int)
{
    return false;
}

bool CHardwareInstance::Query (const std::string &query, void *)
{
    // We do not support the query specified.
    TraceMessage (ttError, "unsupported query: '%s'", 
            query.c_str ());
    return false;
}

// ====================================================================
// Method Name : CharToDump
// Description : This method will format character data into a hexadecimal
//                formatted output string.
// Parameters  : szData - character data to show in hexadecimal format
//               nDataLen - length of szData
//               szReturnBuffer - character buffer to hold the returned hexadecimal output
//               nReturnBufferLen - length of szReturnBuffer
// Returns     : szReturnBuffer
// ====================================================================
std::string CHardwareInstance::CharToDump (const unsigned char *data, 
        size_t len) const
{
    static const char hexDigits[] = "0123456789ABCDEF";
    std::string result;
    char byteString[4] = "xx ";
    size_t i;
    
    if (len == 0 || len == -1)
        len = std::strlen ((const char *) data);

    for (i = 0; i < len; ++i)
    {
        byteString[0] = hexDigits[(data[i] >> 4) & 0x0f];
        byteString[1] = hexDigits[data[i] & 0x0f];
        result += byteString;
    }
    
    return result;
}


// ====================================================================
// Method Name : GetLastErrorAsString
// Description : Display error message text, given an error code.
//                Typically, the parameter passed to this function is retrieved
//                from GetLastError().
// Returns     : Pointer to static buffer containing the error.  This method is
//                not thread safe, since the return buffer is shared by all threads.
// ====================================================================
std::string CHardwareInstance::GetLastErrorAsString (void) const
{
   char  szErrorBuffer[ 512 ] = "(unknown error)";

   FormatMessage(
     FORMAT_MESSAGE_FROM_SYSTEM,
     NULL,
     GetLastError(),
     0,
     szErrorBuffer,
     511,
     NULL );

   return std::string (szErrorBuffer);
}


// ====================================================================
// Method Name : CheckForServerVersion
// Description : This method will check to see if the server version is at
//                or above the version values passed.
// Parameters  : nMajor - Integer that specifies the required major version number
//             : nMinor - Integer that specifies the required minor version number
//             : nSubMinor - Integer that specifies the required subminor version number.
// Returns     : TRUE if version is >= values passed.  FALSE if version is < values passed.
// ====================================================================
bool CHardwareInstance::CheckForServerVersion( int nMajor, int nMinor, int nSubMinor ) const
{
   int   nMaj = 0;
   int   nMin = 0;
   int   nSubMin = 0;

   if (3 != sscanf( m_cbi.m_szHBVersion, "%d.%d.%d", &nMaj, &nMin, &nSubMin ))
   {
      if (2 != sscanf( m_cbi.m_szHBVersion, "%d.%d", &nMaj, &nMin ))
      {
         return( FALSE );
      }
   }

   //
   // Test the values.
   if (nMaj > nMajor)
   {
      return( TRUE );
   }

   if ((nMaj == nMajor) && (nMin > nMinor))
   {
      return( TRUE );
   }

   if ((nMaj == nMajor) && (nMin == nMinor) && (nSubMin >= nSubMinor))
   {
      return( TRUE );
   }

   return( FALSE );
}

///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
// Call back helper wrappers.  See HardwareAPI.h for description
// of the methods and their parameters.
///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
bool CHardwareInstance::RegisterHardwareModuleProperty( 
        const std::string &propertyName, 
        P_TYPE pType, 
        const char** aValues, 
        const std::string &defaultValue, 
        bool showInMainConfig, 
        const std::string &wizardText)
{
    CMFCStateSwitcher switcher;
    if ( m_cbi.pHB_RegisterHardwareModuleProperty2( m_instance, 
            propertyName.c_str (), 
            pType, 
            aValues, 
            defaultValue.c_str (), 
            showInMainConfig, 
            wizardText.empty () ? 
                (const char *) NULL : wizardText.c_str ()))
        return true;
    return false;
}

bool CHardwareInstance::SetPropertyLimits( 
        const std::string &propertyName, 
        int nLowLimit, 
        int nHighLimit )
{
    CMFCStateSwitcher switcher;
    if( m_cbi.pHB_SetPropertyLimits( m_instance, 
            propertyName.c_str (), 
            nLowLimit, 
            nHighLimit ))
        return true;
    return false;
}

bool CHardwareInstance::RegisterHardwareModuleButton( 
        int buttonId, 
        const std::string &buttonHeading,
        const std::string &buttonText)
{
    CMFCStateSwitcher switcher;
    if ( m_cbi.pHB_RegisterHardwareModuleButton( m_instance, 
            buttonId, 
            buttonHeading.c_str (), 
            buttonText.c_str ()) )
        return true;
    return false;
}

std::string CHardwareInstance::GetPropertyNameFromHandle( 
    PROPERTY_VALUE_HANDLE handle) const
{
    CMFCStateSwitcher switcher;
    const char *result = m_cbi.pHB_GetPropertyNameFromHandle (handle);
    if (!result)
        return std::string ();
    return std::string (result);
}

std::string CHardwareInstance::GetPropertyValueFromName( 
    const std::string &propertyName ) const
{
    CMFCStateSwitcher switcher;
    const char *result = m_cbi.pHB_GetPropertyValueFromName (m_instance,
        propertyName.c_str ());

    if (!result)    
        return std::string ();
    return std::string (result);
}

std::string CHardwareInstance::GetPropertyValueFromHandle( 
        PROPERTY_VALUE_HANDLE handle) const
{
    CMFCStateSwitcher switcher;
    const char *result = m_cbi.pHB_GetPropertyValueFromHandle (handle);
    if (!result)
        return std::string ();
    return std::string (result);
}

bool CHardwareInstance::GetPropertyInfoAtIndex( 
        int index, 
        std::string *propertyName, 
        std::string *propertyValue ) const
{
    CMFCStateSwitcher switcher;
    char nameString[256], valueString[256];
    if (!m_cbi.pHB_GetPropertyInfoAtIndex (m_instance, index, nameString, 
            valueString))
        return false;

    propertyName->assign (nameString);
    propertyValue->assign (valueString);
    return true;
}

bool CHardwareInstance::SetPropertyValueFromName( 
        const std::string &propertyName, 
        const std::string &propertyValue )
{
    CMFCStateSwitcher switcher;
    if (m_cbi.pHB_SetPropertyValueFromName( m_instance, 
            propertyName.c_str (), 
            propertyValue.c_str ()))
        return true;
    return false;
}

bool CHardwareInstance::SetPropertyNameAndValueAtIndex( 
        int index, 
        const std::string &propertyName, 
        const std::string &propertyValue )
{
    CMFCStateSwitcher switcher;
    if (m_cbi.pHB_SetPropertyNameAndValueAtIndex (m_instance,
            index, propertyName.c_str (), propertyValue.c_str ()))
        return true;
    return false;
}

bool CHardwareInstance::DeletePropertyValueByNameAndValue( 
        const std::string &propertyName, 
        const std::string &propertyValue )
{
    CMFCStateSwitcher switcher;
    if (m_cbi.pHB_DeletePropertyValueByNameAndValue (m_instance,
            propertyName.c_str (), propertyValue.c_str ()))
        return true;
    return false;
}

bool CHardwareInstance::DeletePropertyValueByIndex( 
        int index)
{
    CMFCStateSwitcher switcher;
    
    if (m_cbi.pHB_DeletePropertyValueByIndex (m_instance, index))
        return true;
    return false;
}

void CHardwareInstance::TraceMessage( 
        TRC_TYPE ttType, const char* szMessage, ... ) const
{
   char  szFormattedMessage[ 1024 ];

   //
   // Format the message
   va_list args;
   va_start( args, szMessage );
   int nLen = _vsnprintf( szFormattedMessage, 1023, szMessage, args );
   if (nLen == -1)
   {
      nLen = sizeof( szFormattedMessage ) - 1;
   }
   szFormattedMessage[ nLen ] = 0;

   //
   // Send it to the server.
   CMFCStateSwitcher switcher;
   m_cbi.pHB_TraceMessage( m_instance, ttType, szFormattedMessage );
}

void CHardwareInstance::NotifySubscribedDevices( 
        const std::string &listName, 
        const std::string &filter, 
        const CDataPack* dataPack)
{
    TraceMessage( ttDebug, 
        "Sending Subscription Notification for list [%s]", listName.c_str ());

    CMFCStateSwitcher switcher;
    m_cbi.pHB_NotifySubscribedDevices (
            listName.c_str (), 
            filter.empty () ? NULL : filter.c_str (), 
            dataPack);
}

bool CHardwareInstance::GetIRCodes (std::vector< IRCode > *codes) const
{
    CMFCStateSwitcher switcher;
    
    codes->clear ();
    DWORD needed = 0;
    if (m_cbi.pHB_GetIRCodes (NULL, &needed))
    {
        codes->resize (needed / sizeof (IRCode));
        if (m_cbi.pHB_GetIRCodes (&(*codes->begin ()), &needed))
            return true;
    }
    return false;
}

bool CHardwareInstance::GetIRCodeFromPosition( 
        unsigned int usIRPosition, 
        IRCode* pCode ) const
{
    CMFCStateSwitcher switcher;
    if ( m_cbi.pHB_GetIRCodeFromPosition( usIRPosition, pCode ) )
        return true;
    return false;
}

bool CHardwareInstance::IsRemoteControlDescriptionUnique( 
    const std::string &remoteDescription ) const
{
    CMFCStateSwitcher switcher;
    if ( m_cbi.pHB_IsRemoteControlDescriptionUnique( 
            remoteDescription.c_str () ) )
        return true;
    return false;
}

bool CHardwareInstance::GetRemoteControlList( 
        std::vector< RemoteControl > *controls) const
{
    CMFCStateSwitcher switcher;
    DWORD needed = 0;
    if (m_cbi.pHB_GetRemoteControlList (NULL, &needed))
    {
        controls->resize (needed / sizeof (RemoteControl));
        if (m_cbi.pHB_GetRemoteControlList (&(*controls->begin ()), &needed))
            return true;
    }
    
    return false;
}

bool CHardwareInstance::AddRemoteControl( 
        const std::string &remoteDescription, 
        int frequency, 
        const std::string &manufacturer, 
        const std::string &model )
{
    CMFCStateSwitcher switcher;
    if (m_cbi.pHB_AddRemoteControl( 
            remoteDescription.c_str (), 
            frequency, 
            manufacturer.c_str (), 
            model.c_str ()))
        return true;
    return false;
}

bool CHardwareInstance::AddOrUpdateIRCode( 
    IRCode* pCode, 
    bool bNewCode )
{
    CMFCStateSwitcher switcher;
    if ( m_cbi.pHB_AddOrUpdateIRCode( 
            pCode, 
            bNewCode ? TRUE : FALSE) )
        return true;
    return false;
}

bool CHardwareInstance::RemoveIRCode( 
        const IRCode* pCode )
{
    CMFCStateSwitcher switcher;
    if ( m_cbi.pHB_RemoveIRCode( pCode ) )
        return true;
    return false;
}

int CHardwareInstance::DoesDeviceExist( 
        const std::string &deviceName, 
        bool exactMatch, 
        const std::string &deviceDescription, 
        const std::string &propertyName, 
        const std::string &propertyValue ) const
{
    CMFCStateSwitcher switcher;
    return( m_cbi.pHB_DoesDeviceExist( 
            deviceName.c_str (), 
            exactMatch, 
            deviceDescription.c_str (), 
            propertyName.c_str (), 
            propertyValue.c_str ()) );
}

int CHardwareInstance::CreateDevice( 
        const std::string &deviceName, 
        const std::string &deviceDescription, 
        const std::string &deviceComment, 
        HARDWARE_INSTANCE_HANDLE hInstance )
{
    CMFCStateSwitcher switcher;
    return( m_cbi.pHB_CreateDevice( 
            deviceName.c_str (), 
            deviceDescription.c_str (), 
            deviceComment.c_str (), 
            hInstance ) );
}

bool CHardwareInstance::SetDeviceProperty( 
        int deviceId, 
        const std::string &propertyName, 
        const std::string &propertyValue )
{
    CMFCStateSwitcher switcher;
    if ( m_cbi.pHB_SetDeviceProperty( 
            deviceId, 
            propertyName.c_str (), 
            propertyValue.c_str ()) )
        return true;
    return false;
}

