// ====================================================================
// Original file Copyright (c) 2002 - 2004 Meedio, LLC.  All rights reserved.
//    This file is provided as part of a software development kit (SDK).
//    This file can be used, royalty free, in any form to extend the
//    functionality of the HouseBot product.

#include "stdafx.h"
#include "DataPack.h"

#include <cstdlib>
#include <algorithm>
#include <functional>

// ====================================================================
// Method Name : CDataPack Constructor
// Parameters  : None
// Description : CDataPack Constructor
// Returns     : None
// ====================================================================
CDataPack::CDataPack()
{
    std::fill (&m_dataNames[0], &m_dataNames[kMaxValues], (char *) NULL);
    std::fill (&m_dataValues[0], &m_dataValues[kMaxValues], (BYTE *) NULL);
    std::fill (&m_dataLens[0], &m_dataLens[kMaxValues], 0);
}

// ====================================================================
// Method Name : CDataPack Destructor
// Parameters  : None
// Description : CDataPack Destructor
// Returns     : None
// ====================================================================
CDataPack::~CDataPack()
{
    int n;
    for (n = 0; n < kMaxValues; ++n)
    {
        delete[] m_dataNames[n];
        delete[] m_dataValues[n];
    }
}

/*
 * locate dataName in m_dataNames, but stop searching if a NULL slot is
 * found.  Returns NULL if full.
 */
static bool findStringOrNull (const char *s1, const char *s2)
{
    if (s1 == NULL || std::strcmp (s1, s2) == 0)
        return true;
    return false;
}

const char *const *CDataPack::findData (const char *dataName) const
{
    const char *const *data = std::find_if (&m_dataNames[0], 
        &m_dataNames[kMaxValues], std::bind2nd (std::ptr_fun (
        findStringOrNull), dataName));
    if (data >= &m_dataNames[kMaxValues])
        return NULL;
    return data;
}

// ====================================================================
// Method Name : AddData
// Parameters  : szDataName - Name of data to add.  szDataValue - Value of data
// Description : This method will add a named string data element to the pack.
// Returns     : TRUE if added successfully.  FALSE on error
// ====================================================================
bool CDataPack::AddData (const char *dataName, const char *dataValue)
{
    const char * const* data = findData (dataName);

    if (!data || *data)
        return false;

    size_t n = (data - &m_dataNames[0]);
    
    m_dataNames[n] = new char[strlen (dataName)+1];
    std::strcpy (m_dataNames[n], dataName);
    
    size_t len = strlen (dataValue)+1;
    m_dataLens[n] = (int) len;
    m_dataValues[n] = new BYTE[len];
    std::strcpy ((char *)m_dataValues[n], dataValue);

    return true;
}


   // ====================================================================
   // Method Name : AddData - For adding binary data
   // Parameters  : szDataName - Name of data to add.
	// 				: pDataValue - Data value
	// 				: nDataSize - Size of Data in pDataValue
   // Description : This method will add a named binary data element to the pack.
   // Returns     : TRUE if added successfully.  FALSE on error
   // ====================================================================
bool CDataPack::AddData (const char *dataName, const void *dataValue, int dataSize)
{
    const char * const* data = findData (dataName);

    if (!data || *data)
        return false;

    size_t n = (data - &m_dataNames[0]);
    
    m_dataNames[n] = new char[strlen (dataName)+1];
    std::strcpy (m_dataNames[n], dataName);
    
    int len = m_dataLens[n] = dataSize;
    m_dataValues[n] = new BYTE[len];
    std::memcpy (m_dataValues[n], dataValue, dataSize);

    return true;
}


// ====================================================================
// Method Name : GetData
// Parameters  : szDataName - Name of data to return value.
// Description : This method will return the string data that corresponds to the data name specified
// Returns     : Value of data specified by szDataName.  "" If szDataName is invalid
// ====================================================================
const char *CDataPack::GetData (const char *dataName) const
{
    const char * const* data = findData (dataName);
    if (!data || !*data)
        return "";
    
    size_t n = (data - &m_dataNames[0]);
    return (const char *) m_dataValues[n];
}

// ====================================================================
// Method Name : GetBinaryData - For reading binary data
// Parameters  : szDataName - Name of data to return value.
// 				: nDataSize - Reference that contains the size of the data returned.
// Description : This method will return the binary data that corresponds to the data name specified
// Returns     : Value of data specified by szDataName.  NULL If szDataName is invalid.
// ====================================================================
const void *CDataPack::GetBinaryData (const char *dataName, int *dataSize) const
{
    const char * const* data = findData (dataName);
    if (!data || !*data)
        return NULL;

    size_t n = (data - &m_dataNames[0]);

    if (dataSize) *dataSize = m_dataLens[n];
    return m_dataValues[n];
}
